/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

//------------------------------------------------------------------------------
//* Display operation:
//*
//* This module controls the display functionality based on settings retrieved
//* from a table stored in either ROM or NVM.  The module can be configured for
//* use in firmware builds supporting a supervisory micro, in which case the
//* function calls are coded in such a fashion that "local" control of the LEDs
//* is not conducted when the presence of a supervisor is detected.
//*
//* The programmer should configure the mapping file for appropriate operation,
//* including the specification of the LED control table location, and the number
//* of control options available.
//*
//------------------------------------------------------------------------------

//---------------------------
// STANDARD LIBRARY INCLUDES
//---------------------------


//----------------------
// APPLICATION INCLUDES
//----------------------
#include "peripheralInit.h"
#include "display.h"
#include "defines.h"
#include "wct_hal.h"
#include "wct_hal_cfg.h"
//-----------------
// PRIVATE DEFINES
//-----------------
void DisplayUpdateLocalTimers( WORD wTimePassedMs );

//-------------------
// PRIVATE VARIABLES
//-------------------
#if LED_DIMMING_ENABLED
const BYTE bLedDimmingControl[ NUM_LED_PIN_ASSIGNMENT_OPTIONS ][ NUM_LED_CHANNELS ][ NUM_LEDS_PER_CHANNEL ] = LED_DIMMING_CONTROL;
#if TMS320F280XX
volatile WORD * const ptrLedDimmingPwmPeriodPorts[ NUM_LEDS_PER_CHANNEL ] = LED_DIMMING_PWM_PERIOD_PORTS;
volatile WORD * const ptrLedDimmingPwmDutyCyclePorts[ NUM_LEDS_PER_CHANNEL ] = LED_DIMMING_PWM_DUTY_CYCLE_PORTS;
volatile WORD * const ptrLedDimmingPwmEnablePorts[ NUM_LEDS_PER_CHANNEL ] = LED_DIMMING_PWM_ENABLE_PORTS;
#else
volatile STANDARD_REGISTER_SIZE * const ptrLedDimmingPwmPeriodPorts[ NUM_LEDS_PER_CHANNEL ] = LED_DIMMING_PWM_PERIOD_PORTS;
volatile STANDARD_REGISTER_SIZE * const ptrLedDimmingPwmDutyCyclePorts[ NUM_LEDS_PER_CHANNEL ] = LED_DIMMING_PWM_DUTY_CYCLE_PORTS;
volatile STANDARD_REGISTER_SIZE * const ptrLedDimmingPwmEnablePorts[ NUM_LEDS_PER_CHANNEL ] = LED_DIMMING_PWM_ENABLE_PORTS;
#endif
STANDARD_REGISTER_SIZE const LedDimmingPwmEnableValues[ NUM_LEDS_PER_CHANNEL ] = LED_DIMMING_PWM_ENABLE_VALUES;
STANDARD_REGISTER_SIZE const LedDimmingPwmDisableValues[ NUM_LEDS_PER_CHANNEL ] = LED_DIMMING_PWM_DISABLE_VALUES;
#endif

STANDARD_REGISTER_SIZE * const ptrLedPort[ NUM_LED_PIN_ASSIGNMENT_OPTIONS ][ NUM_LED_CHANNELS ][ NUM_LEDS_PER_CHANNEL ] = LED_PORTS;
const STANDARD_REGISTER_SIZE LedBitMask[ NUM_LED_PIN_ASSIGNMENT_OPTIONS ][ NUM_LED_CHANNELS ][ NUM_LEDS_PER_CHANNEL ] = LED_BIT_MASKS;
const WORD wLedPinAssignment[ NUM_LED_BINS ] = LED_PIN_ASSIGNMENTS;
const BOOL bActiveHighLedDrive[ NUM_LED_PIN_ASSIGNMENT_OPTIONS ] = LED_PIN_ASSIGNMENT_ACTIVE_HIGH;
const BOOL bLedControlViaNVM[ NUM_LED_BINS ] = LED_CONTROL_VIA_NVM_INIT;

// LED matrix informs the firmware how to control the LEDs
// NOTE: Bin '0' is reserved for operation using NVM parameters.
// Organization:
//  LED1_ON, LED1_Blink, LED2_ON, LED2_Blink, LED3_ON, LED3_Blink, Fault Blink Rate, MOD Fault Blink Rate, Op Blink Rate, Pwr-Up Delay
const LED_OPERATION LedMatrix[ NUM_LED_BINS ] = LED_MATRIX_INITIALIZATION;
LED_OPERATION* pLedControlParams = ( LED_OPERATION* )&NvmParams.SystemParams.LedOperation;
#if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
BYTE byChargeStatus;
WORD wFaultBlinkAccum = 0;
WORD wModFaultBlinkAccum = 0;
WORD wOpStateBlinkAccum = 0;
#endif
#if LOW_POWER_MODE_SUPPORTED
static WORD wLowPowerModeTimeMs;
#endif

#if SIMPLIFY_DISPLAY_CONTROL

typedef struct {
    DEVICE_STATE eLedDevState; // Device state
    WORD wLedFastBlinkAccTimeMs; //Record how long is passed for fast blink LED state.
    WORD wLedSlowBlinkAccTimeMs; //Record how long is passed for slow blink LED state.
}T_LED_CONTROL_STRUCTURE;

#define LED_SLOW_BLINK_DURATION 2000    // millisecond
#define LED_FAST_BLINK_DURATION 1000    // millisecond

#define LED_STATES_BITS 4
#define LED_STATES_BIT_MASK 0xf

#define LED_1_BIT_POSITION 0
#define LED_2_BIT_POSITION 4
#define LED_3_BIT_POSITION 8
#define LED_4_BIT_POSITION 12

#define LED_1_BIT_MASK 0x000f
#define LED_2_BIT_MASK 0x00f0
#define LED_3_BIT_MASK 0x0f00
#define LED_4_BIT_MASK 0xf000

#define LED_BIT_TO_WORD(led1, led2, led3, led4) ((led1<<LED_1_BIT_POSITION)+(led2<<LED_2_BIT_POSITION)+(led3<<LED_3_BIT_POSITION)+(led4<<LED_4_BIT_POSITION))

//FSL WCT1000 A11 EVM two LED: LED1,LED2
//FSL WCT1000 A11 EVM LED defines       ----------------LED1------------LED2------------LED3------------LED4------------
#define LED_STATE_INITIALIZING          LED_BIT_TO_WORD(LED_ON,         LED_ON,         LED_OFF,        LED_OFF)
#define LED_STATE_DIAGNOSTIC            LED_BIT_TO_WORD(LED_BLINK_SLOW, LED_OFF,        LED_OFF,        LED_OFF)
#define LED_STATE_NO_DEVICE_PRESENT     LED_BIT_TO_WORD(LED_OFF,        LED_BLINK_SLOW, LED_OFF,        LED_OFF)                                         
#define LED_STATE_DEVICE_CHARGING       LED_BIT_TO_WORD(LED_BLINK_SLOW, LED_ON,         LED_OFF,        LED_OFF)
#define LED_STATE_DEVICE_CHARGED        LED_BIT_TO_WORD(LED_OFF,        LED_ON,         LED_OFF,        LED_OFF)
#define LED_STATE_DEVICE_FOD_FAULT      LED_BIT_TO_WORD(LED_ON,         LED_OFF,        LED_OFF,        LED_OFF)
#define LED_STATE_DEVICE_FAULT          LED_BIT_TO_WORD(LED_ON,         LED_OFF,        LED_OFF,        LED_OFF)
#define LED_STATE_SYSTEM_FAULT          LED_BIT_TO_WORD(LED_ON,         LED_OFF,        LED_OFF,        LED_OFF)
#define LED_STATE_NVM_FAULT             LED_BIT_TO_WORD(LED_ON,         LED_OFF,        LED_OFF,        LED_OFF)
#define LED_STATE_POWER_LIMIT           LED_BIT_TO_WORD(LED_BLINK_FAST, LED_BLINK_FAST, LED_OFF,        LED_OFF)


#if 0
//Customer - LED: Red/Green/Blue: Led1/2/3
//Customer - LED defines mode 1         ----------------LED1------------LED2------------LED3------------LED4------------     
#define LED_STATE_INITIALIZING          LED_BIT_TO_WORD(LED_ON,         LED_ON,         LED_ON,         LED_OFF)
#define LED_STATE_DIAGNOSTIC            LED_BIT_TO_WORD(LED_BLINK_SLOW, LED_OFF,        LED_OFF,        LED_OFF)
#define LED_STATE_NO_DEVICE_PRESENT     LED_LOOP_BLINK_SLOW                                             
#define LED_STATE_DEVICE_CHARGING       LED_BIT_TO_WORD(LED_OFF,        LED_ON,         LED_OFF,        LED_OFF)
#define LED_STATE_DEVICE_CHARGED        LED_BIT_TO_WORD(LED_OFF,        LED_BLINK_SLOW, LED_OFF,        LED_OFF)
#define LED_STATE_DEVICE_FOD_FAULT      LED_BIT_TO_WORD(LED_ON,         LED_OFF,        LED_OFF,        LED_OFF)
#define LED_STATE_DEVICE_FAULT          LED_BIT_TO_WORD(LED_ON,         LED_OFF,        LED_OFF,        LED_OFF)
#define LED_STATE_SYSTEM_FAULT          LED_BIT_TO_WORD(LED_BLINK_FAST, LED_OFF,        LED_OFF,        LED_OFF)
#define LED_STATE_NVM_FAULT             LED_BIT_TO_WORD(LED_BLINK_FAST, LED_OFF,        LED_OFF,        LED_OFF)
#define LED_STATE_POWER_LIMIT           LED_BIT_TO_WORD(LED_BLINK_FAST, LED_BLINK_FAST, LED_OFF,        LED_OFF)
#endif

WORD LedStatesDefTable1[10] = {
    LED_STATE_INITIALIZING,     
    LED_STATE_DIAGNOSTIC,       
    LED_STATE_NO_DEVICE_PRESENT,
    LED_STATE_DEVICE_CHARGING,  
    LED_STATE_DEVICE_CHARGED,   
    LED_STATE_DEVICE_FOD_FAULT, 
    LED_STATE_DEVICE_FAULT,                                     
    LED_STATE_SYSTEM_FAULT,     
    LED_STATE_NVM_FAULT,        
    LED_STATE_POWER_LIMIT,      
};

#if LED_LOOP_BLINK
// Customer - Foxconn LED loop blink under standby mode
BYTE LedLoopBlinkTable_Standby[LED_LOOP_BLINK_COUNTS] = {
    0x01, // R
    0x02, // G
    0x04, // B
    0x03, // R/G
    0x05, // R/B
    0x06, // B/G
};
#endif

WORD *pLedStatesDefTable = LedStatesDefTable1;

STANDARD_REGISTER_SIZE * const ptrCustomerLedPort[ NUM_LED_CHANNELS ][ CUSTOMER_NUM_LEDS_PER_CHANNEL ] = CUSTOMER_LED_PORTS;
const STANDARD_REGISTER_SIZE CustomerLedBitMask[ NUM_LED_CHANNELS ][ CUSTOMER_NUM_LEDS_PER_CHANNEL ] = CUSTOMER_LED_BIT_MASKS;


T_LED_CONTROL_STRUCTURE LedCtrlTable[NUM_LED_CHANNELS]; //Led Control table. 
#if LED_LOOP_BLINK
WORD wLedLoopBlinkCnt;
#endif
#endif

#if LOW_POWER_DISPLAY_MODE
struct 
{
	WORD  wLedOnTimeInLowPowerModeMs;
	WORD  wLedOffTimeInLowPowerModeMs;
	DWORD dwLAccuTimeInLowPowerModeMs;
} LedStInLowPowerMode;
#endif

//-------------------
// PRIVATE FUNCTIONS
//-------------------

#if SIMPLIFY_DISPLAY_CONTROL
void Led_Loop_Control(BYTE byLedChannel, WORD wTimePassedMs)
{
    WORD LedState, i;
    BOOL fastBlk;
    BOOL slowBlk;
    T_LED_CONTROL_STRUCTURE *pLedCtrlTbl;

    pLedCtrlTbl = &LedCtrlTable[byLedChannel];
    LedState = pLedStatesDefTable[pLedCtrlTbl->eLedDevState];

    if(pLedCtrlTbl->wLedFastBlinkAccTimeMs >= wTimePassedMs)
    {
        pLedCtrlTbl->wLedFastBlinkAccTimeMs -= wTimePassedMs;
        fastBlk=FALSE;
    }
    else
    {
        pLedCtrlTbl->wLedFastBlinkAccTimeMs = LED_FAST_BLINK_DURATION;
        fastBlk=TRUE;
    }

    if(pLedCtrlTbl->wLedSlowBlinkAccTimeMs >= wTimePassedMs)
    {
        pLedCtrlTbl->wLedSlowBlinkAccTimeMs -= wTimePassedMs;
        slowBlk=FALSE;
    }
    else
    {
        pLedCtrlTbl->wLedSlowBlinkAccTimeMs = LED_SLOW_BLINK_DURATION;
        slowBlk=TRUE;
    }

#if LED_LOOP_BLINK
    if ( LedState == LED_LOOP_BLINK_SLOW || LedState == LED_LOOP_BLINK_FAST)
    {
        WORD LedLoopBlinkNum;
        BYTE LedLoopBlinkMask;
        
        LedLoopBlinkNum = wLedLoopBlinkCnt%LED_LOOP_BLINK_COUNTS;
        LedLoopBlinkMask = LedLoopBlinkTable_Standby[LedLoopBlinkNum];
        if ( (LedState == LED_LOOP_BLINK_SLOW && slowBlk == TRUE) ||
            (LedState == LED_LOOP_BLINK_FAST && fastBlk == TRUE) )
            wLedLoopBlinkCnt++;

        for(i=0;i<CUSTOMER_NUM_LEDS_PER_CHANNEL;i++)
        {
            if((LedLoopBlinkMask>>i)&0x01)
            {    // LED on
                #if (CUSTOMER_LED_PIN_ASSIGNMENT_ACTIVE_HIGH==0)   // Active low
                *ptrCustomerLedPort[byLedChannel][i] &= ~ CustomerLedBitMask[byLedChannel][i];
                #else
                *ptrCustomerLedPort[byLedChannel][i] |=    CustomerLedBitMask[byLedChannel][i];
                #endif
            }
            else
            {    // LED off
                #if (CUSTOMER_LED_PIN_ASSIGNMENT_ACTIVE_HIGH==0)   // Active low
                *ptrCustomerLedPort[byLedChannel][i] |=    CustomerLedBitMask[byLedChannel][i];
                #else
                *ptrCustomerLedPort[byLedChannel][i] &= ~ CustomerLedBitMask[byLedChannel][i];
                #endif
            }
        }
    }
    else
#endif
    {
        for(i=0;i<CUSTOMER_NUM_LEDS_PER_CHANNEL;i++)
        {
            switch (LedState & LED_STATES_BIT_MASK)
            {
                case LED_ON:
                    #if (CUSTOMER_LED_PIN_ASSIGNMENT_ACTIVE_HIGH==0)   // Active low
                    *ptrCustomerLedPort[byLedChannel][i] &= ~ CustomerLedBitMask[byLedChannel][i];
                    #else
                    *ptrCustomerLedPort[byLedChannel][i] |=    CustomerLedBitMask[byLedChannel][i];
                    #endif
                    break;
                case LED_OFF:
                    #if (CUSTOMER_LED_PIN_ASSIGNMENT_ACTIVE_HIGH==0)   // Active low
                    *ptrCustomerLedPort[byLedChannel][i] |=    CustomerLedBitMask[byLedChannel][i];
                    #else
                    *ptrCustomerLedPort[byLedChannel][i] &= ~ CustomerLedBitMask[byLedChannel][i];
                    #endif
                    break;
                case LED_BLINK_SLOW:
                    if(slowBlk)
                        *ptrCustomerLedPort[byLedChannel][i] ^=    CustomerLedBitMask[byLedChannel][i];
                    break;
                case LED_BLINK_FAST:
                    if(fastBlk)
                        *ptrCustomerLedPort[byLedChannel][i] ^=    CustomerLedBitMask[byLedChannel][i];
                    break;
                default:
                    break;
            }
        
            LedState >>= LED_STATES_BITS;
        }
    }

}
#endif

#if LED_DIMMING_ENABLED
//------------------------------------------------------------------------------
static void DimLed( WORD wLedNum, SWORD swLedIntensity )
{
    if( swLedIntensity == LED_DIMMING_INTENSITY_OFF )
    {
        // enable the pwm output
        *ptrLedDimmingPwmEnablePorts[ wLedNum ] = LedDimmingPwmDisableValues[ wLedNum ];
    }
    else
    {
        DWORD dwLedDutyCycle;
        DWORD dwMin;
        DWORD dwMax;
        DWORD dwPwmDutyCycle;
        DWORD dwPwmPeriod;

        dwPwmPeriod = *ptrLedDimmingPwmPeriodPorts[ wLedNum ];

        // set the min and max pwm duty cycle from NVM
        dwMin = NvmParams.SystemParams.LedOperation.LedParams[ wLedNum ].LedDimmingParams.wPwmDutyCycleMin;
        dwMax = NvmParams.SystemParams.LedOperation.LedParams[ wLedNum ].LedDimmingParams.wPwmDutyCycleMax;

        // convert the led intensity to the led duty cycle
        // note: assumes LED_DIMMING_INTENSITY_MIN is 0
        dwLedDutyCycle = ( dwMax - dwMin );
        dwLedDutyCycle *= swLedIntensity;
        dwLedDutyCycle /= LED_DIMMING_INTENSITY_MAX;
        dwLedDutyCycle += dwMin;

        // convert the the led duty cycle to the pwm duty cycle 
        dwPwmDutyCycle = dwLedDutyCycle * dwPwmPeriod;
        dwPwmDutyCycle /= LED_DIMMING_INTENSITY_MAX;

        // if the led drive is active high, invert the pwm duty cycle
        if( HwConfigOptions.bActiveHighLedDrive )
            dwPwmDutyCycle = dwPwmPeriod - dwPwmDutyCycle;

        // set the pwm duty cycle           
        *ptrLedDimmingPwmDutyCyclePorts[ wLedNum ] = dwPwmDutyCycle;

        // enable the pwm output
        *ptrLedDimmingPwmEnablePorts[ wLedNum ] = LedDimmingPwmEnableValues[ wLedNum ];
    }
}
#endif

#if !SIMPLIFY_DISPLAY_CONTROL
//------------------------------------------------------------------------------
static void TurnLedOn( WORD wPinAssignment, BYTE byLedChannel, WORD wLedNum )
{
    #if LED_DIMMING_ENABLED
    BYTE LedDimmingControl = bLedDimmingControl[ wPinAssignment ][ byLedChannel ][ wLedNum ];

    if( LedDimmingControl == LED_DIMMING_CONTROL_VIA_APP )
        DimLed( wLedNum, LedIntensity[ wLedNum ] );
    else if( LedDimmingControl == LED_DIMMING_CONTROL_VIA_PIN )
    {
        if( LED_DIMMIMG_CONTROL_BIT )
            DimLed( wLedNum, LED_DIMMING_INTENSITY_MAX );
        else
            DimLed( wLedNum, LED_DIMMING_INTENSITY_MIN );
    }
    else
    #endif
    {
        STANDARD_REGISTER_SIZE* pControlReg = ptrLedPort[ wPinAssignment ][ byLedChannel ][ wLedNum ];
        STANDARD_REGISTER_SIZE BitMask = LedBitMask[ wPinAssignment ][ byLedChannel ][ wLedNum ];

        if( HwConfigOptions.bActiveHighLedDrive )
            *pControlReg |= BitMask;
        else
            *pControlReg &= ~BitMask;
    }
}

//------------------------------------------------------------------------------
static void TurnLedOff( WORD wPinAssignment, BYTE byLedChannel, WORD wLedNum )
{
    #if LED_DIMMING_ENABLED
    BOOL LedDimmingControl = bLedDimmingControl[ wPinAssignment ][ byLedChannel ][ wLedNum ];
        
    if( LedDimmingControl == LED_DIMMING_CONTROL_VIA_APP || LedDimmingControl == LED_DIMMING_CONTROL_VIA_PIN )
    {
        DimLed( wLedNum, LED_DIMMING_INTENSITY_OFF );
    }
    else
    #endif
    {
        STANDARD_REGISTER_SIZE* pControlReg = ptrLedPort[ wPinAssignment ][ byLedChannel ][ wLedNum ];
        STANDARD_REGISTER_SIZE BitMask = LedBitMask[ wPinAssignment ][ byLedChannel ][ wLedNum ];

        if( HwConfigOptions.bActiveHighLedDrive )
            *pControlReg &= ~BitMask;
        else
            *pControlReg |= BitMask;
    }
}

#if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
//------------------------------------------------------------------------------
void DisplayUpdateLocalTimers( WORD wTimePassedMs )
{
    // Bump all blink accumulators
    if( wTimePassedMs > ( pLedControlParams->wModFaultBlinkRateMs - wModFaultBlinkAccum ) )
        wModFaultBlinkAccum = 0;
    else
        wModFaultBlinkAccum += wTimePassedMs;

    if( wTimePassedMs > ( pLedControlParams->wFaultBlinkRateMs - wFaultBlinkAccum ) )
        wFaultBlinkAccum = 0;
    else
        wFaultBlinkAccum += wTimePassedMs;

    if( wTimePassedMs > ( pLedControlParams->wOpStateBlinkRateMs - wOpStateBlinkAccum ) )
        wOpStateBlinkAccum = 0;
    else
        wOpStateBlinkAccum += wTimePassedMs;
}
#endif

//------------------------------------------------------------------------------
#if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
static void DisplayUpdateLedStatusStructure( BYTE byLedChannel, WORD wMask, DEVICE_STATE eDeviceState, BYTE byChargeStatus )
#else
static void DisplayUpdateLedStatusStructure( BYTE byLedChannel, WORD wMask, DEVICE_STATE eDeviceState )
#endif
{
    int i;
    LED_CONTROL* pLedControl = &LedControl[ byLedChannel ][ 0 ];
    #if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
    DWORD dwTemp;
    #endif

    // Service each LED for the specified device
    for( i = 0; i < NUM_LEDS_PER_CHANNEL; i++, pLedControl++ )
    {
        if( pLedControlParams->LedParams[i].wLedOnOffStateBitfield.all & wMask )
        {
            // This LED should be on
            pLedControl->wLedStatus = LED_ON;
        }
        else if( pLedControlParams->LedParams[i].wLedBlinkStateBitfield.all & wMask )
        {
            // This LED should be blinking
            pLedControl->wLedStatus = LED_BLINK;

            // Check LED Blink time
            switch( eDeviceState )
            {
                #if FOD_SUPPORTED
                case DEVICE_FOD_FAULT:
                #else
                case DEVICE_MOD_FAULT:
                #endif
                    #if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
                    pLedControl->wDutyCycleMs = ( pLedControlParams->wModFaultBlinkRateMs >> 1 );
                    pLedControl->eBlinkType = BLINK_TYPE_MOD_FAULT;
                    #else
                    pLedControl->wBlinkTimeMs = pLedControlParams->wModFaultBlinkRateMs;
                    #endif
                    break;
                case DEVICE_FAULT:
                case SYSTEM_FAULT:
                case NVM_FAULT:
                case POWER_LIMIT:
                    #if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
                    pLedControl->wDutyCycleMs = ( pLedControlParams->wFaultBlinkRateMs >> 1 );
                    pLedControl->eBlinkType = BLINK_TYPE_FAULT;
                    #else
                    pLedControl->wBlinkTimeMs = pLedControlParams->wFaultBlinkRateMs;
                    #endif
                    break;
                #if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
                case DEVICE_CHARGING:

                    pLedControl->eBlinkType = BLINK_TYPE_OP_STATE;

                    if( pLedControlParams->LedParams[i].wLedBlinkStateBitfield.bit.btLedDisplayChargeState )
                    {
                        // Blink at a duty cycle equivalent to the charge status

                        // Check for valid Charge Status
                        // NOTE: The byChargeStatus should be limited between '0' and '100'
                        if( byChargeStatus == 255 )
                        {
                            // Assume 0% charged
                            byChargeStatus = 0;
                        }
                        else if( byChargeStatus > 100 )
                        {
                            // Truncate to 100%
                            byChargeStatus = 100;
                        }

                        if( pLedControlParams->LedParams[i].wLedBlinkStateBitfield.bit.btLedInvertChargeStateDisplay )
                        {
                            // The indication is inverted
                            dwTemp = ( 100 - byChargeStatus );
                            dwTemp *= pLedControlParams->wOpStateBlinkRateMs;
                            dwTemp /= 100;
                            pLedControl->wDutyCycleMs = (WORD) dwTemp;
                        }
                        else
                        {
                            // The indication is not inverted
                            dwTemp = byChargeStatus;
                            dwTemp *= pLedControlParams->wOpStateBlinkRateMs;
                            dwTemp /= 100;
                            pLedControl->wDutyCycleMs = (WORD) dwTemp;
                        }
                    }
                    else
                    {
                        // Blink at a 50% duty cycle
                        pLedControl->wDutyCycleMs = ( pLedControlParams->wOpStateBlinkRateMs >> 1 );
                    }
                    break;
                #endif
                default:
                    #if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
                    pLedControl->eBlinkType = BLINK_TYPE_OP_STATE;
                    pLedControl->wDutyCycleMs = ( pLedControlParams->wOpStateBlinkRateMs >> 1 );
                    #else
                    pLedControl->wBlinkTimeMs = pLedControlParams->wOpStateBlinkRateMs;
                    #endif
                    break;
            }
        }
        else
        {
            // This LED should be OFF
            pLedControl->wLedStatus = LED_OFF;
        }
    }

#if (NUM_LEDS_PER_CHANNEL==2)
    if( LedControl[ byLedChannel ][ 0 ].wLedStatus == LED_BLINK && LedControl[ byLedChannel ][ 1 ].wLedStatus == LED_BLINK)
    {   // Let LED1&2 blink at same time
        for( i = 0; i < NUM_LEDS_PER_CHANNEL; i++)
        {
            LedControl[ byLedChannel ][ i ].byLedState = 0;
            LedControl[ byLedChannel ][ i ].wAccumTimeMs = 0;
        }
    }
#endif
}
#endif //#if !SIMPLIFY_DISPLAY_CONTROL

//------------------------------------------------------------------------------
static void DisplayUpdateLocalLeds( BYTE byLedChannel, WORD wTimePassedMs )
{
#if SIMPLIFY_DISPLAY_CONTROL
    Led_Loop_Control(byLedChannel, wTimePassedMs);
#else
    WORD wLedNum;
    WORD wPinAssignment = 0;
    LED_CONTROL* pLedControl = &LedControl[ byLedChannel ][ 0 ];
    #if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
    WORD wBlinkAccum;
    #if LOW_POWER_MODE_SUPPORTED
    WORD wTimeToNextBlink;
    #endif
    #endif

    // Calculate index for LED pin assignment
    wPinAssignment = wLedPinAssignment[ HwConfigOptions.byLedSelection ];

    // Service each LED for the specified device
    for( wLedNum = 0; wLedNum < NUM_LEDS_PER_CHANNEL; wLedNum++, pLedControl++ )
    {
        switch( pLedControl->wLedStatus )
        {
            case LED_BLINK:
                #if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
                // Toggle LED at specified duty of the blink time
                switch( pLedControl->eBlinkType )
                {
                    case BLINK_TYPE_MOD_FAULT:
                        wBlinkAccum = wModFaultBlinkAccum;
                        break;
                    case BLINK_TYPE_FAULT:
                        wBlinkAccum = wFaultBlinkAccum;
                        break;
                    case BLINK_TYPE_OP_STATE:
                        wBlinkAccum = wOpStateBlinkAccum;
                        break;
                    default:
                        wBlinkAccum = 0;
                        break;
                }

                if( wBlinkAccum < pLedControl->wDutyCycleMs )
                {
                    // LED should be ON
                    TurnLedOn( wPinAssignment, byLedChannel, wLedNum );
                    #if LOW_POWER_MODE_SUPPORTED
                    wTimeToNextBlink = ( pLedControl->wDutyCycleMs - wBlinkAccum );
                    #endif
                }
                else
                {
                    // LED should be OFF
                    TurnLedOff( wPinAssignment, byLedChannel, wLedNum );
                    #if LOW_POWER_MODE_SUPPORTED
                    wTimeToNextBlink = ( ( pLedControl->wDutyCycleMs << 1 ) - wBlinkAccum );
                    #endif
                }

                #if LOW_POWER_MODE_SUPPORTED
                // Assess next blink time to calculate low power mode time
                if( wTimeToNextBlink < wLowPowerModeTimeMs )
                {
                    wLowPowerModeTimeMs = wTimeToNextBlink;
                }
                #endif

                #else
                // Toggle LED at 1/2 the blink time (50% duty cycle)

                if( pLedControl->wAccumTimeMs >= wTimePassedMs )
                    pLedControl->wAccumTimeMs -= wTimePassedMs;
                else
                    pLedControl->wAccumTimeMs = 0;

                if( pLedControl->wAccumTimeMs == 0 )
                {
                    // Reload the accumulator for 50% duty cycle
                    pLedControl->wAccumTimeMs = ( pLedControl->wBlinkTimeMs >> 1 );

                    // Toggle the LED pin
                    if( pLedControl->byLedState )
                    {
                        // We are currently "ON"...
                        pLedControl->byLedState = 0;
                        TurnLedOff( wPinAssignment, byLedChannel, wLedNum );
                    }
                    else
                    {
                        // We are currently "OFF"...
                        pLedControl->byLedState = 1;
                        TurnLedOn( wPinAssignment, byLedChannel, wLedNum );
                    }
                }
                #if LOW_POWER_MODE_SUPPORTED
                // Assess next blink time to calculate low power mode time
                if( pLedControl->wAccumTimeMs < wLowPowerModeTimeMs )
                {
                    wLowPowerModeTimeMs = pLedControl->wAccumTimeMs;
                }
                #endif

                #endif
                break;
            case LED_ON:
                #if !CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
                pLedControl->wAccumTimeMs = 0;
                pLedControl->byLedState = 1;
                #endif
                TurnLedOn( wPinAssignment, byLedChannel, wLedNum );
                break;
            case LED_OFF:
            default:
                #if !CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
                pLedControl->wAccumTimeMs = 0;
                pLedControl->byLedState = 0;
                #endif
                TurnLedOff( wPinAssignment, byLedChannel, wLedNum );
                break;
        }
    }
#endif
}

//------------------
// PUBLIC VARIABLES
//------------------
// NOTE: This array needs to be accessible by the supervisor module
LED_CONTROL LedControl[ NUM_LED_CHANNELS ][ NUM_LEDS_PER_CHANNEL ];

#if LED_DIMMING_ENABLED
SWORD LedIntensity[ NUM_LEDS_PER_CHANNEL ] = LED_DIMMING_INTENSITY_INITIALIZATION;
#endif

//------------------
// PUBLIC FUNCTIONS
//------------------


//------------------------------------------------------------------------------
void DisplayInit( void )
{
    BYTE byLedChannel;

    // NOTE: This function should ONLY be called when we are in the 
    //  INITIALIZATION state!  (Performs any power-up delay specified.)
    for( byLedChannel = 0; byLedChannel < NUM_LED_CHANNELS; byLedChannel++ )
    {
        #if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
        DisplayUpdateLedStatus( byLedChannel, INITIALIZING, 0 );
        #else
        DisplayUpdateLedStatus( byLedChannel, INITIALIZING );
        #endif

        DisplayUpdateLocalLeds( byLedChannel, 0 );
    }

    // Perform any LED Power-up delay if value is valid
    // (Prevent 65 second delay due to erased data flash)
    if( pLedControlParams->wDelayAtPowerUpMs != 0xFFFF )
    {
        WCT_TimerDelayMs( pLedControlParams->wDelayAtPowerUpMs );
    }

    // Update LEDs with "STANDBY" indication
    for( byLedChannel = 0; byLedChannel < NUM_LED_CHANNELS; byLedChannel++ )
    {
        #if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
        DisplayUpdateLedStatus( byLedChannel, NO_DEVICE_PRESENT, 0 );
        #else
        DisplayUpdateLedStatus( byLedChannel, NO_DEVICE_PRESENT );
        #endif

        DisplayUpdateLocalLeds( byLedChannel, 0 );
    }
#if LOW_POWER_DISPLAY_MODE
    LedStInLowPowerMode.dwLAccuTimeInLowPowerModeMs = 0U;
    LedStInLowPowerMode.wLedOnTimeInLowPowerModeMs = LED_ON_TIME_IN_LOW_POWER_MODE_MS; 
    LedStInLowPowerMode.wLedOffTimeInLowPowerModeMs = LED_OFF_TIME_IN_LOW_POWER_MODE_MS;
#endif
}


//------------------------------------------------------------------------------
void DisplayInitLedControl( void )
{
#if SIMPLIFY_DISPLAY_CONTROL
    WORD i;

    for(i=0;i<NUM_LED_CHANNELS;i++)
    {
        LedCtrlTable[i].eLedDevState = INITIALIZING;
        LedCtrlTable[i].wLedFastBlinkAccTimeMs = LED_FAST_BLINK_DURATION;
        LedCtrlTable[i].wLedSlowBlinkAccTimeMs = LED_SLOW_BLINK_DURATION;
    }
#else
    // Insure we have a valid index into the matrix
    if( HwConfigOptions.byLedSelection >= NUM_LED_BINS )
        HwConfigOptions.byLedSelection = ( NUM_LED_BINS - 1 );

    // Check LED configuration
    if( bLedControlViaNVM[ HwConfigOptions.byLedSelection ] )
    {
        // Utilize the parameters specified in NVM
    }
    else
    {
        // Utilize the selection requested from ROM
        pLedControlParams = ( LED_OPERATION* )&LedMatrix[ HwConfigOptions.byLedSelection ];
    }

    // Initialize ActiveHighLedDrive flag
    HwConfigOptions.bActiveHighLedDrive = bActiveHighLedDrive[ wLedPinAssignment[ HwConfigOptions.byLedSelection ] ];
#endif
}


//------------------------------------------------------------------------------
#if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
void DisplayUpdateLedStatus( BYTE byLedChannel, DEVICE_STATE eDeviceState, BYTE byChargeStatus )
#else
void DisplayUpdateLedStatus( BYTE byLedChannel, DEVICE_STATE eDeviceState )
#endif
{
#if SIMPLIFY_DISPLAY_CONTROL
    if (LedCtrlTable[byLedChannel].eLedDevState != eDeviceState)
    {
        LedCtrlTable[byLedChannel].eLedDevState = eDeviceState;
        LedCtrlTable[byLedChannel].wLedFastBlinkAccTimeMs=LED_FAST_BLINK_DURATION;
        LedCtrlTable[byLedChannel].wLedSlowBlinkAccTimeMs=LED_SLOW_BLINK_DURATION;
        ResetLedGpio(byLedChannel);	// Reset LED GPIO state.
#if LED_LOOP_BLINK
        if( pLedStatesDefTable[eDeviceState]  == LED_LOOP_BLINK_SLOW || 
            pLedStatesDefTable[eDeviceState] == LED_LOOP_BLINK_FAST )
            wLedLoopBlinkCnt = 0;
#endif
    }
#else
    WORD wMask;

    // Determine mask and any blink rates
    switch ( eDeviceState )
    {
        case INITIALIZING:
            wMask = INITIALIZING_BITMASK;
            break;

        case NO_DEVICE_PRESENT:
#if LOW_POWER_MODE_SUPPORTED
	#if LOW_POWER_DISPLAY_MODE
            wMask = 0U;
	#else
			wMask = STANDBY_BITMASK;
	#endif
#else
			wMask = STANDBY_BITMASK;
#endif
            break;

        case DEVICE_CHARGING:
            wMask = POWER_TRANSFER_BITMASK;
            break;

        case DEVICE_CHARGED:
            wMask = DEVICE_CHARGED_BITMASK;
            break;

        #if FOD_SUPPORTED
        case DEVICE_FOD_FAULT:
        #else
        case DEVICE_MOD_FAULT:
        #endif
            wMask = MOD_FAULT_BITMASK;
            break;

        case DEVICE_FAULT:
            wMask = DEVICE_FAULT_BITMASK;
            break;

        case SYSTEM_FAULT:
            wMask = SYSTEM_FAULT_BITMASK;
            break;

        case NVM_FAULT:
            wMask = NVM_MEMORY_FAULT_BITMASK;
            break;

        case POWER_LIMIT:
            wMask = POWER_LIMIT_BITMASK;
            break;
            
        case DIAGNOSTIC_MODE:
            wMask = DIAG_LED_ON_BITMASK;
            break;
            
        default:
            // Illegal device status
            wMask = 0;
            break;
    }

    #if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
    DisplayUpdateLedStatusStructure( byLedChannel, wMask, eDeviceState, byChargeStatus );
    #else
    DisplayUpdateLedStatusStructure( byLedChannel, wMask, eDeviceState );
    #endif

    //ResetLedGpio(byLedChannel);	// Reset LED GPIO state.
#endif

}


//------------------------------------------------------------------------------
#if LOW_POWER_MODE_SUPPORTED
WORD DisplayProcessTimeSlice( WORD wTimePassedMs )
#else
void DisplayProcessTimeSlice( WORD wTimePassedMs )
#endif
{
    BYTE byLedChannel;

    #if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
    DisplayUpdateLocalTimers( wTimePassedMs );
    #endif

    #if LOW_POWER_MODE_SUPPORTED
    wLowPowerModeTimeMs = LOW_POWER_MODE_TIME_INFINITE;
    #endif

    for( byLedChannel = 0; byLedChannel < NUM_LED_CHANNELS; byLedChannel++ )
    {
        DisplayUpdateLocalLeds( byLedChannel, wTimePassedMs );
    }

    #if LOW_POWER_MODE_SUPPORTED
    return wLowPowerModeTimeMs;
    #endif
}

//------------------------------------------------------------------------------
void ResetLedGpio( BYTE byLedChannel )
{
    WORD i;
    #if !SIMPLIFY_DISPLAY_CONTROL
    STANDARD_REGISTER_SIZE* pControlReg;
    STANDARD_REGISTER_SIZE BitMask;
    #endif
    
    for(i=0;i<CUSTOMER_NUM_LEDS_PER_CHANNEL;i++)
    {
        #if SIMPLIFY_DISPLAY_CONTROL
        #if (CUSTOMER_LED_PIN_ASSIGNMENT_ACTIVE_HIGH==0)   // Active low
        *ptrCustomerLedPort[byLedChannel][i] |=    CustomerLedBitMask[byLedChannel][i];
        #else
        *ptrCustomerLedPort[byLedChannel][i] &= ~ CustomerLedBitMask[byLedChannel][i];
        #endif
        #else
        LedControl[ byLedChannel ][ i ].byLedState = 0;
        LedControl[ byLedChannel ][ i ].wAccumTimeMs = 0;
        #endif
    }
}
#if 0
//------------------------------------------------------------------------------
void DisplayDiagnosticCoilLed( BYTE byLedChannel, BYTE byLedIndex, BOOL bEnable )
{
    // NOTE: This function should only be called by the diagnostics, it assumes
    //      the main DisplayProcessTimeSlice function is not being run.

    if( NUM_LEDS_PER_CHANNEL > byLedIndex )
    {
        // Calling function is requesting specific control of an LED
        if( bEnable )
        {
            LedControl[ byLedChannel ][ byLedIndex ].wLedStatus = LED_ON;
        }
        else
        {
            LedControl[ byLedChannel ][ byLedIndex ].wLedStatus = LED_OFF;
        }
    }
    else
    {
        // Calling function is requesting update of all LEDs in the channel
        if( bEnable )
        {
            // Send Supervisory status to turn on LEDs as specified in NVM
            #if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
            DisplayUpdateLedStatusStructure( byLedChannel, DIAG_LED_ON_BITMASK, DIAGNOSTIC_MODE, 0 );
            #else
            DisplayUpdateLedStatusStructure( byLedChannel, DIAG_LED_ON_BITMASK, DIAGNOSTIC_MODE );
            #endif
        }
        else
        {
            // Send Supervisory status to turn on LEDs as specified in NVM
            #if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
            DisplayUpdateLedStatusStructure( byLedChannel, DIAG_LED_OFF_BITMASK, DIAGNOSTIC_MODE, 0 );
            #else
            DisplayUpdateLedStatusStructure( byLedChannel, DIAG_LED_OFF_BITMASK, DIAGNOSTIC_MODE );
            #endif
        }
    }

    DisplayUpdateLocalLeds( byLedChannel, 0 );
}
#endif

#if LOW_POWER_DISPLAY_MODE
/***************************************************************************************
 *  Func Name: DisplayUpdateLedsInStandby
 *  Description: Update Leds' status in low Power mode
 *  Parameters:  wStopTimeMs: time in which MCU is in STOP mode
 *               wTimePassedMs: Time passed
 *  Return:  None               
 ****************************************************************************************/
void DisplayUpdateLedsInLowPowerMode( WORD wStopTimeMs,WORD wTimePassedMs)
{
	
	LedStInLowPowerMode.dwLAccuTimeInLowPowerModeMs += wStopTimeMs;//accumulate stop times
	LedControl[0][0].wLedStatus = LED_OFF;
	LedControl[0][1].wLedStatus = LED_OFF;
	
	if(LedStInLowPowerMode.dwLAccuTimeInLowPowerModeMs <= LedStInLowPowerMode.wLedOffTimeInLowPowerModeMs)
	{
		LedControl[0][LED_PORT_BLINK_IN_LOW_POWER_MODE].wLedStatus = LED_OFF;
	}
	else
	{
		LedControl[0][LED_PORT_BLINK_IN_LOW_POWER_MODE].wLedStatus = LED_ON;
		if((LedStInLowPowerMode.dwLAccuTimeInLowPowerModeMs - LedStInLowPowerMode.wLedOffTimeInLowPowerModeMs) > LedStInLowPowerMode.wLedOnTimeInLowPowerModeMs )
		{
			LedControl[0][LED_PORT_BLINK_IN_LOW_POWER_MODE].wLedStatus = LED_OFF;
			LedStInLowPowerMode.dwLAccuTimeInLowPowerModeMs = 0; //roll over
		}
	}
	DisplayProcessTimeSlice(wTimePassedMs);
}
#endif
