/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/
//---------------------------
// STANDARD LIBRARY INCLUDES
//---------------------------


//----------------------
// APPLICATION INCLUDES
//----------------------
//#include "uController.h"            // NOTE: This MUST occur before the inclusion of mapping.h for proper operation
#include "sound.h"
#include "pwm.h"
#include "defines.h"


#if SOUND_SUPPORTED

//-----------------
// PRIVATE DEFINES
//-----------------

typedef enum
{
    SOUND_STATE_IDLE = 0,
    SOUND_STATE_FIRST_TONE,
    SOUND_STATE_FIRST_PAUSE,
    SOUND_STATE_SECOND_TONE,
    SOUND_STATE_SECOND_PAUSE,
    MAX_SOUND_STATES

} SOUND_STATE;



//-------------------
// PRIVATE VARIABLES
//-------------------
static SOUND_PARAMS* pSoundParams = &gSoundParams;
#if AC_SOUND_ENABLED
SOUND_STATE eSoundState;
SOUND_TYPES eSoundRequest = SOUND_NONE;
WORD wFirstFrequency;
WORD wSecondFrequency;
WORD wSoundTimer;
#endif
#if DC_SOUND_ENABLED
WORD wDcBeeperTimer = 0;
#endif

//-------------------
// PRIVATE FUNCTIONS
//-------------------
void SoundPwmFrequencyUpdate(WORD wFrequency);

#if AC_SOUND_ENABLED
//------------------------------------------------------------------------------
void SoundSetFrequency( WORD wFrequency )
{
    SoundPwmFrequencyUpdate(wFrequency);
}

void SoundPwmFrequencyUpdate(WORD wFrequency)
{
    WORD wPWMVal1;
    WORD wPWMVal3;
    
    // Calculate the normal PWM Modulus/period value required for the specified frequency
    wPWMVal1    = (WORD)SOUND_CALC_PERIOD_VALUE_FROM_FREQ( wFrequency );
    wPWMVal3    = wPWMVal1>>1;   // Duty cycle is fixed to 50%

    PWMA_MCTRL |= PWMA_MCTRL_CLDOK_2;

    PWMA_SM2INIT = 0;
    PWMA_SM2VAL1 = ( WORD )wPWMVal1;
    PWMA_SM2VAL4 = 0;
    PWMA_SM2VAL5 = ( WORD )wPWMVal3;

    PWMA_MCTRL |= PWMA_MCTRL_LDOK_2;

}
#endif


//------------------
// PUBLIC VARIABLES
//------------------


//------------------
// PUBLIC FUNCTIONS
//------------------


//------------------------------------------------------------------------------
void SoundInit( void )
{
    // Initialize the queue request, flush the state machine
    eSoundRequest = SOUND_NONE;

    #if AC_SOUND_ENABLED
    DISABLE_SOUND_PWM_OUTPUT;
    eSoundState = SOUND_STATE_IDLE;
    wSoundTimer = 0;
    #endif

    #if DC_SOUND_ENABLED
    PIN_DC_BEEPER_LOW;
    wDcBeeperTimer = 0;
    #endif

    #if PWM0_USED_FOR_SOUND_CONTROL
    Pwm0Init( 0 );
    #endif
    #if PWM1_USED_FOR_SOUND_CONTROL
    Pwm1Init( 0 );
    #endif
    #if PWM2_USED_FOR_SOUND_CONTROL
    Pwm2Init( 0 );
    #endif
    #if PWM3_USED_FOR_SOUND_CONTROL
    Pwm3Init( 0 );
    #endif
}


//------------------------------------------------------------------------------
#if LOW_POWER_MODE_SUPPORTED
WORD SoundProcessTimeSlice( WORD wTimePassedMs )
#else
void SoundProcessTimeSlice( WORD wTimePassedMs )
#endif
{
    #if LOW_POWER_MODE_SUPPORTED
    WORD wLowPowerModeTimeMs = LOW_POWER_MODE_TIME_INFINITE;
    #endif

    #if DC_SOUND_ENABLED
    // Service the DC beeper timer
    if( wDcBeeperTimer )
    {
        // Output is on, decrement the timer
        if( wDcBeeperTimer >= wTimePassedMs )
            wDcBeeperTimer -= wTimePassedMs;
        else
            wDcBeeperTimer = 0;

        if( wDcBeeperTimer == 0 )
        {
            // Timer has expired, turn off output
            PIN_DC_BEEPER_LOW;
        }
        #if LOW_POWER_MODE_SUPPORTED
        else
        {
            wLowPowerModeTimeMs = wDcBeeperTimer;
        }
        #endif
    }
    #endif

    #if AC_SOUND_ENABLED
    // Service the AC beeper - Decrement counter if we can
    if( wSoundTimer >= wTimePassedMs )
        wSoundTimer -= wTimePassedMs;
    else
        wSoundTimer = 0;

    if( wSoundTimer == 0 )
    {
        // Check operating point
        switch( eSoundState )
        {
            case SOUND_STATE_FIRST_TONE:
                // We are done with first frequency, turn off output
                DISABLE_SOUND_PWM_OUTPUT;

                // Set first pause duration
                wSoundTimer = pSoundParams->wSoundFirstPauseDuration;
                eSoundState = SOUND_STATE_FIRST_PAUSE;
                break;

            case SOUND_STATE_FIRST_PAUSE:
                // We are done with first pause, output second frequency
                SoundSetFrequency( wSecondFrequency );

                ENABLE_SOUND_PWM_OUTPUT;

                // Set second tone duration
                wSoundTimer = pSoundParams->wSoundSecondToneDuration;
                eSoundState = SOUND_STATE_SECOND_TONE;
                break;

            case SOUND_STATE_SECOND_TONE:
                // We are done with second frequency, turn off output
                DISABLE_SOUND_PWM_OUTPUT;

                // Set second pause duration
                wSoundTimer = pSoundParams->wSoundSecondPauseDuration;
                eSoundState = SOUND_STATE_SECOND_PAUSE;
                break;

            case SOUND_STATE_SECOND_PAUSE:
                // We are done with second pause, get ready for next queue item
                eSoundState = SOUND_STATE_IDLE;
                break;

            case SOUND_STATE_IDLE:
            default:
                // Look for new request in queue
                if( eSoundRequest )
                {
                    // Service the next item in the queue
                    if( eSoundRequest == SOUND_POWER_ON )
                    {
                        // output LOW frequency first
                        wFirstFrequency = pSoundParams->wSoundLowFrequency;
                        wSecondFrequency = pSoundParams->wSoundHighFrequency;
                    }
                    else
                    {
                        // output HIGH frequency first
                        wFirstFrequency = pSoundParams->wSoundHighFrequency;
                        wSecondFrequency = pSoundParams->wSoundLowFrequency;
                    }

                    // Reset request
                    eSoundRequest = SOUND_NONE;

                    // Output the first frequency
                    SoundSetFrequency( wFirstFrequency );

                    ENABLE_SOUND_PWM_OUTPUT;

                    // Set first tone duration
                    wSoundTimer = pSoundParams->wSoundFirstToneDuration;
                    eSoundState = SOUND_STATE_FIRST_TONE;
                }
                break;
        }
    }

    #if LOW_POWER_MODE_SUPPORTED
    if( ( wSoundTimer != 0 ) && ( wSoundTimer < wLowPowerModeTimeMs ) )
    {
        wLowPowerModeTimeMs = wSoundTimer;
    }
    #endif

    #endif  // End if AC_SOUND_ENABLED

    #if LOW_POWER_MODE_SUPPORTED
    return wLowPowerModeTimeMs;
    #endif
}


//------------------------------------------------------------------------------
void SoundQueueUpdate( SOUND_TYPES eSoundType )
{
    // Abort any pending sound generation
    #if AC_SOUND_ENABLED
    DISABLE_SOUND_PWM_OUTPUT;
    #endif
    #if DC_SOUND_ENABLED
    PIN_DC_BEEPER_LOW;
    #endif

    #if AC_SOUND_ENABLED
    // Reset state machine
    eSoundState = SOUND_STATE_IDLE;
    wSoundTimer = 0;

    // Set new request
    eSoundRequest = eSoundType;
    #endif


    #if DC_SOUND_ENABLED
    // The DC beeper is only used when powering ON
    if( eSoundType == SOUND_POWER_ON )
    {
        PIN_DC_BEEPER_HIGH;
        wDcBeeperTimer = pSoundParams->wDcBeeperDuration;
    }
    else
    {
        wDcBeeperTimer = 0;
    }
    #endif

}

#endif




