/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

// NOTE: The QSCI0 in the UCD30x0 devices does NOT contain a receive FIFO

//---------------------------
// STANDARD LIBRARY INCLUDES
//---------------------------


//----------------------
// APPLICATION INCLUDES
//----------------------
#include "wct_callbacks.h"
#include "wct_hal.h"
#include "wct_hal_cfg.h"
#include "peripheralInit.h"
#include "wct_libconst.h"
#include "qsci.h"

//-----------------
// PRIVATE DEFINES
//-----------------
QSCI0_TX_INTERRUPT;

//-------------------
// PRIVATE VARIABLES
//-------------------

volatile BOOL bRxEnabled = FALSE;
#if QSCI0_TX_INTERRUPT_DRIVEN
volatile BYTE byQSCI0TxBuffReadIndex;                      // Index into TX buffer of next byte to read
volatile BYTE byQSCI0TxBuffWriteIndex;                     // Index into TX buffer of next byte to write
volatile BYTE byQSCI0TxBuffer[ QSCI0_TX_BUFFER_SIZE ];     // Uart Transmit buffer
#define IS_TRANSMITTING 1
volatile BYTE byQSCI0TxStatus;

#endif

#if QSCI1_TX_INTERRUPT_DRIVEN
volatile BYTE byQSCI1TxBuffReadIndex;                      // Index into TX buffer of next byte to read
volatile BYTE byQSCI1TxBuffWriteIndex;                     // Index into TX buffer of next byte to write
volatile BYTE byQSCI1TxBuffer[ QSCI1_TX_BUFFER_SIZE ];     // Uart Transmit buffer
#define IS_TRANSMITTING 1
volatile BYTE byQSCI1TxStatus;

#endif



//-------------------
// PRIVATE FUNCTIONS
//-------------------

#if QSCI0_ENABLED
#if QSCI0_TX_INTERRUPT_DRIVEN
//------------------------------------------------------------------------------
#if CODE_WARRIOR
#pragma interrupt saveall
#endif
QSCI0_TX_INTERRUPT
{
    // QSCI0 transmitter empty interrupt
    // The transmit data register is available to receive another character

    // Clear the TX empty interrupt flag by reading the flag, then writing another character to the data register
    volatile WORD st = QSCI0_STAT;
    
    if( st & QSCI0_STAT_TDRE )
    {
        if( byQSCI0TxBuffReadIndex == byQSCI0TxBuffWriteIndex )
        {
            // This was our last transmission...
            // If IrDA is enabled and loopback is disabled, we need to prevent reception of our own data
            #if QSCI0_IRDA_INTERFACE
            if( !( QSCI0_CTRL1 & LOOP_MODE_ENABLED ) )
            {
                if( bRxEnabled )
                {
                    // Re-enable the receiver
                    ENABLE_QSCI0_RX;
                }
            }
            #endif
            #if DEBUG_CONSOLE
            DEBUG_PIN_4_TOGGLE;
            #endif
            byQSCI0TxStatus &= ~IS_TRANSMITTING;
            //DISABLE_QSCI0_TX_EMPTY_INTERRUPTS
            QSCI0_CTRL1 &= ~QSCI0_CTRL1_TEIE;
        }
        else
        {
            // Fill the Transmitter as long as we have data
            while( ( st&QSCI0_STAT_TDRE ) && ( byQSCI0TxBuffReadIndex != byQSCI0TxBuffWriteIndex ) )
            {
                // Send the next character to the FIFO, increment the read index
                QSCI0_DATA = byQSCI0TxBuffer[ byQSCI0TxBuffReadIndex ];
                byQSCI0TxBuffReadIndex++;

                // Roll over the index if it is larger than the buffer
                if ( byQSCI0TxBuffReadIndex >= QSCI0_TX_BUFFER_SIZE )
                {
                    byQSCI0TxBuffReadIndex = 0;
                }
                st = QSCI0_STAT;
            }
        }
    }
}
#if CODE_WARRIOR
#pragma interrupt off
#endif
#endif


#if QSCI0_RX_INTERRUPT_ENABLED
//------------------------------------------------------------------------------
#pragma interrupt
void QSCI0RxInterrupt( void )
{
    // NOTE: This interrupt should be used to read error status and handle the error accordingly
    // Disable interrupts during ISR to prevent spurious interrupts.
    DISABLE_QSCI0_RX_INTERRUPTS;

    // Clear all receive error flags by reading the status register then writing anything to the STAT register
    // This is done by the following read-modify-write command.
    wStatusReg = QSCI0_STAT;
    QSCI0_STAT = 1;

    // TODO: handle the error outside of the ISR

    // Re-enable the RX Interrupts
    ENABLE_QSCI0_RX_INTERRUPTS;
}
#endif


//------------------
// PUBLIC VARIABLES
//------------------


//------------------
// PUBLIC FUNCTIONS
//------------------




//------------------------------------------------------------------------------
WORD Qsci0GetStatus( void )
{
    return QSCI0_STAT;
}

//------------------------------------------------------------------------------

#if DEBUG_CONSOLE_QSCI0
void Qsci0PutChar( BYTE byChar )
{
#if QSCI0_TX_INTERRUPT_DRIVEN

    BYTE bySpacesAvail;
    BOOL bTxIntSetting;
    
    if( QSCI0_CTRL1 & QSCI0_CTRL1_TE )
    {
        // Tx is enabled...

        #if DEBUG_CONSOLE
        DEBUG_PIN_2_TOGGLE;
        #endif
        #if QSCI0_IRDA_INTERFACE
        // If we are in IrDA mode, we need to prevent reception of our own transmission
        bRxEnabled = QSCI0_CTRL1 & ENABLE_RX;
        DISABLE_QSCI0_RX;
        #endif
        
        bTxIntSetting = QSCI0_CTRL1 & QSCI0_CTRL1_TEIE;            // Save off the current interrupt setting
        //when TX fifo is full, write character to buffer instead to data reg.
        if ( !(QSCI0_STAT & QSCI0_STAT_TDRE) )
            byQSCI0TxStatus = IS_TRANSMITTING;
        
        // See if we need to fill the transmitter (do this to prevent getting stuck if the ISR fails to fill the buffer)
        if( (byQSCI0TxBuffReadIndex == byQSCI0TxBuffWriteIndex) && (!(byQSCI0TxStatus&IS_TRANSMITTING)) )
        {
           	// Send the next character to the transmitter
           	QSCI0_DATA = byChar;
        }
        else
        {
            // Enable TX interrupts first so we can clear the circular buffer and not 'hang'
        	 QSCI0_CTRL1 |= QSCI0_CTRL1_TEIE;

        	//write character to buffer
        	byQSCI0TxBuffer[ byQSCI0TxBuffWriteIndex ] = byChar;
            byQSCI0TxBuffWriteIndex++;
#if DEBUG_CONSOLE
            DEBUG_PIN_3_TOGGLE;
#endif
        	// Roll over the index if it is larger than the buffer
             if ( byQSCI0TxBuffWriteIndex >= QSCI0_TX_BUFFER_SIZE )
             {
            	 byQSCI0TxBuffWriteIndex = 0;
             }
        }
        QSCI0_CTRL1 |= bTxIntSetting;
        // Wait until transmit buffer is ready - can become a blocking function!
        //while( QSCI0_CTRL2 & TX_FIFO_COUNT_BITMASK );

    }
#else

    if( QSCI0_CTRL1 & ENABLE_TX )
    {
        // Tx is enabled...
        //DISABLE_QSCI0_RX_INTERRUPTS;
        //DISABLE_QSCI0_TX_EMPTY_INTERRUPTS;

        #if QSCI0_IRDA_INTERFACE
        // Disable the Receiver to prevent reception of our own transmission
        bRxEnabled = QSCI0_CTRL1 & ENABLE_RX;
        DISABLE_QSCI0_RX;
        #endif

        // Wait until transmit buffer has room
        while( !( QSCI0_STAT & QSCI_TDRE_BITMASK ) );

        QSCI0_DATA = byChar;

        #if QSCI0_IRDA_INTERFACE
        // If we are using an IrDA interface, wait for transmission completion before
        // re-enabling the receiver to prevent reception of our own transmission

        asm( nop );
        asm( nop );

        // Wait until all characters have been transmitted and receiver is IDLE (idle bit goes low)
        while( !( QSCI0_STAT & QSCI_TIDLE_BITMASK ) );

        if( bRxEnabled )
        {
            // Re-enable the receiver
            ENABLE_QSCI0_RX;
        }
        #endif

        // Note: Interrupt flag is automatically cleared by reading RX register
        //ENABLE_QSCI0_RX_INTERRUPTS;
    }
#endif
}

//------------------------------------------------------------------------------
void Qsci0FlushBuffer( void )
{
#if QSCI0_TX_INTERRUPT_DRIVEN
    while( byQSCI0TxBuffReadIndex != byQSCI0TxBuffWriteIndex );
#else
    //poll mode flush buffer on DebugPutString() or DebugPutChar() call
#endif
}

//------------------------------------------------------------------------------
BYTE Qsci0GetChar( void )
{
    // Gets the next character from the QSCI0 receive buffer - will wait until a character is received if none
    // are available
    BYTE byChar;                                                // Temporary storage of the character received
    BOOL bRxIntSetting;
    #if QSCI0_TX_INTERRUPT_DRIVEN
    BOOL bTxIntSetting;
    #endif

    while ( !( QSCI0_STAT & QSCI0_STAT_RDRF )  ) // Wait for a character/
    {
        // Wait for character to appear in buffer

        // TODO
        // CLEARWACHDOG();
    }

    #if QSCI0_TX_INTERRUPT_DRIVEN
    // Insure the TX interrupt is disabled before grabbing RX Interrupt status...
    //  (prevents us from restoring an invalid RX interrupt enable state)
    bTxIntSetting = QSCI0_CTRL1 & QSCI0_CTRL1_TEIE;            // Save off the current interrupt setting
    QSCI0_CTRL1 &= ~QSCI0_CTRL1_TEIE;
    asm( nop );
    asm( nop );
    #endif
    bRxIntSetting = QSCI0_CTRL1 & QSCI0_CTRL1_RFIE;             // Save off the current interrupt setting
    //DISABLE_QSCI0_RX_INTERRUPTS;
    QSCI0_CTRL1 &= ~QSCI0_CTRL1_RFIE;

    byChar = QSCI0_DATA;         // Read the next character from the buffer

    // Restore interrupt setting(s)
    QSCI0_CTRL1 |= bRxIntSetting;
    #if QSCI0_TX_INTERRUPT_DRIVEN
    QSCI0_CTRL1 |= bTxIntSetting;
    #endif
    return byChar;                                              // Return the character

}

//------------------------------------------------------------------------------
BYTE Qsci0GetRxBytes( void )
{
    // Return the number of bytes in the Rx buffer

    BYTE byNumOfBytes;                                          // Number of bytes in the RX Buffer
    BOOL bRxIntSetting;
    #if QSCI0_TX_INTERRUPT_DRIVEN
    BOOL bTxIntSetting;
    #endif

    #if QSCI0_TX_INTERRUPT_DRIVEN
    // Insure the TX interrupt is disabled before grabbing RX Interrupt status...
    //  (prevents us from restoring an invalid RX interrupt enable state)
    bTxIntSetting = QSCI0_CTRL1 & QSCI0_CTRL1_TEIE;           // Save off the current interrupt setting
    QSCI0_CTRL1 &= ~QSCI0_CTRL1_TEIE;
    asm( nop );
    asm( nop );
    #endif
    bRxIntSetting = QSCI0_CTRL1 & QSCI0_CTRL1_RFIE;            // Save off the current interrupt setting
    //DISABLE_QSCI0_RX_INTERRUPTS;
    QSCI0_CTRL1 &= ~QSCI0_CTRL1_RFIE;

    byNumOfBytes = ( ( QSCI0_CTRL2 & QSCI0_CTRL2_RFCNT ) >> 8 );

    // Restore interrupt setting(s)
    QSCI0_CTRL1 |= bRxIntSetting;
    #if QSCI0_TX_INTERRUPT_DRIVEN
    QSCI0_CTRL1 |= bTxIntSetting;
    #endif
    return byNumOfBytes;
}
#endif

//------------------------------------------------------------------------------
void Qsci0Init( void )
{
    // Initialize the QSCI0 controller
	volatile WORD st;
    // This connects the system clock to the serial peripheral
    //clock enabled in SIM init
	//ENABLE_SCI0_IPBUS_CLK;

    #if QSCI0_TX_INTERRUPT_DRIVEN
    byQSCI0TxBuffReadIndex  = 0;
    byQSCI0TxBuffWriteIndex = 0;
    byQSCI0TxStatus = 0;
    #endif

    // Hold QSCI0 in standby state during configuration
    QSCI0_CTRL1 |= QSCI0_CTRL1_RWU;       // 1=Standby state, 0=normal state

    // Configure baud rate
    QSCI0_RATE = SCI_0_SCIBR_INIT;

    // Configure the remainder of the QSCI0 module
    QSCI0_CTRL1 = SCI_0_SCICR_INIT;
    QSCI0_CTRL2 = SCI_0_SCICR2_INIT;

    // Release QSCI0 from reset state
    QSCI0_CTRL1 &= ~QSCI0_CTRL1_RWU;       // 1=Standby state, 0=normal state

    // Interrupts should be enabled by the application
    #if QSCI0_TX_INTERRUPT_DRIVEN
    st = QSCI0_STAT;
    DEBUG_CONSOLE_INTERRUPT_PRIORITY;
    #endif
}
#endif //QSCI0_ENABLED


#if QSCI1_ENABLED

#if QSCI1_TX_INTERRUPT_DRIVEN
//------------------------------------------------------------------------------
QSCI1_TX_INTERRUPT
{
    // QSCI1 transmitter empty interrupt
    // The transmit data register is available to receive another character

    // Clear the TX empty interrupt flag by reading the flag, then writing another character to the data register
    volatile WORD st = QSCI1_STAT;
    
    if( st & QSCI1_STAT_TDRE )
    {
        if( byQSCI1TxBuffReadIndex == byQSCI1TxBuffWriteIndex )
        {
            // This was our last transmission...
            // If IrDA is enabled and loopback is disabled, we need to prevent reception of our own data
            #if QSCI1_IRDA_INTERFACE
            if( !( QSCI1_CTRL1 && LOOP_MODE_ENABLED ) )
            {
                if( bRxEnabled )
                {
                    // Re-enable the receiver
                    ENABLE_QSCI1_RX;
                }
            }
            #endif
            #if DEBUG_CONSOLE
            DEBUG_PIN_4_TOGGLE;
            #endif
            byQSCI1TxStatus &= ~IS_TRANSMITTING;
            //DISABLE_QSCI0_TX_EMPTY_INTERRUPTS
            QSCI1_CTRL1 &= ~QSCI1_CTRL1_TEIE;
        }
        else
        {
            // Fill the Transmitter as long as we have data
            while( ( st&QSCI1_STAT_TDRE ) && ( byQSCI1TxBuffReadIndex != byQSCI1TxBuffWriteIndex ) )
            {
                // Send the next character to the FIFO, increment the read index
                QSCI1_DATA = byQSCI1TxBuffer[ byQSCI1TxBuffReadIndex ];
                byQSCI1TxBuffReadIndex++;

                // Roll over the index if it is larger than the buffer
                if ( byQSCI1TxBuffReadIndex >= QSCI1_TX_BUFFER_SIZE )
                {
                    byQSCI1TxBuffReadIndex = 0;
                }
                st = QSCI1_STAT;
            }
        }
    }
}
#endif


#if QSCI0_RX_INTERRUPT_ENABLED
//------------------------------------------------------------------------------
#pragma interrupt
void QSCI1RxInterrupt( void )
{
    // NOTE: This interrupt should be used to read error status and handle the error accordingly
    // Disable interrupts during ISR to prevent spurious interrupts.
    DISABLE_QSCI1_RX_INTERRUPTS;

    // Clear all receive error flags by reading the status register then writing anything to the STAT register
    // This is done by the following read-modify-write command.
    wStatusReg = QSCI1_STAT;
    QSCI1_STAT = 1;

    // TODO: handle the error outside of the ISR

    // Re-enable the RX Interrupts
    ENABLE_QSCI1_RX_INTERRUPTS;
}
#endif


//------------------
// PUBLIC VARIABLES
//------------------


//------------------
// PUBLIC FUNCTIONS
//------------------

//------------------------------------------------------------------------------
WORD Qsci1GetStatus( void )
{
    return QSCI1_STAT;
}

#if DEBUG_CONSOLE_QSCI1
//------------------------------------------------------------------------------
void Qsci1PutChar( BYTE byChar )
{
    // Transmit a character

#if QSCI1_TX_INTERRUPT_DRIVEN

    BYTE bySpacesAvail;
    BOOL bTxIntSetting;
    
    if( QSCI1_CTRL1 & QSCI1_CTRL1_TE )
    {
        // Tx is enabled...

        #if DEBUG_CONSOLE
        DEBUG_PIN_2_TOGGLE;
        #endif
        #if QSCI1_IRDA_INTERFACE
        // If we are in IrDA mode, we need to prevent reception of our own transmission
        bRxEnabled = QSCI1_CTRL1 & ENABLE_RX;
        DISABLE_QSCI1_RX;
        #endif
        
        bTxIntSetting = QSCI1_CTRL1 & QSCI1_CTRL1_TEIE;            // Save off the current interrupt setting
        //when TX fifo is full, write character to buffer instead to data reg.
        if ( !(QSCI1_STAT & QSCI1_STAT_TDRE) )
            byQSCI1TxStatus = IS_TRANSMITTING;
        
        // See if we need to fill the transmitter (do this to prevent getting stuck if the ISR fails to fill the buffer)
        if( (byQSCI1TxBuffReadIndex == byQSCI1TxBuffWriteIndex) && (!(byQSCI1TxStatus&IS_TRANSMITTING)) )
        {
           	// Send the next character to the transmitter
           	QSCI1_DATA = byChar;
        }
        else
        {
            // Enable TX interrupts first so we can clear the circular buffer and not 'hang'
        	 QSCI1_CTRL1 |= QSCI1_CTRL1_TEIE;

        	//write character to buffer
        	byQSCI1TxBuffer[ byQSCI1TxBuffWriteIndex ] = byChar;
            byQSCI1TxBuffWriteIndex++;
#if DEBUG_CONSOLE
            DEBUG_PIN_3_TOGGLE;
#endif
        	// Roll over the index if it is larger than the buffer
             if ( byQSCI1TxBuffWriteIndex >= QSCI1_TX_BUFFER_SIZE )
             {
            	 byQSCI1TxBuffWriteIndex = 0;
             }
        }
        QSCI1_CTRL1 |= bTxIntSetting;
        // Wait until transmit buffer is ready - can become a blocking function!
        //while( QSCI0_CTRL2 & TX_FIFO_COUNT_BITMASK );

    }
#else

    if( QSCI1_CTRL1 & QSCI0_CTRL1_TE )
    {
        // Tx is enabled...
        //DISABLE_QSCI1_RX_INTERRUPTS;
        //DISABLE_QSCI1_TX_EMPTY_INTERRUPTS;

        #if QSCI1_IRDA_INTERFACE
        // Disable the Receiver to prevent reception of our own transmission
        bRxEnabled = QSCI1_CTRL1 & ENABLE_RX;
        DISABLE_QSCI1_RX;
        #endif

        // Wait until transmit buffer has room
        while( !( QSCI1_STAT & QSCI0_STAT_TDRE ) );

        QSCI1_DATA = byChar;

        #if QSCI1_IRDA_INTERFACE
        // If we are using an IrDA interface, wait for transmission completion before
        // re-enabling the receiver to prevent reception of our own transmission

        asm( nop );
        asm( nop );

        // Wait until all characters have been transmitted and receiver is IDLE (idle bit goes low)
        while( !( QSCI1_STAT & QSCI_TIDLE_BITMASK ) );

        if( bRxEnabled )
        {
            // Re-enable the receiver
            ENABLE_QSCI1_RX;
        }
        #endif

        // Note: Interrupt flag is automatically cleared by reading RX register
        //ENABLE_QSCI1_RX_INTERRUPTS;
    }
#endif

}

//------------------------------------------------------------------------------
void Qsci1FlushBuffer( void )
{
#if QSCI1_TX_INTERRUPT_DRIVEN
    while( byQSCI1TxBuffReadIndex != byQSCI1TxBuffWriteIndex );
#else
    //poll mode flush buffer on DebugPutString() or DebugPutChar() call
#endif
}


//------------------------------------------------------------------------------
BYTE Qsci1GetChar( void )
{
    // Gets the next character from the QSCI1 receive buffer - will wait until a character is received if none
    // are available
    BYTE byChar;                                                // Temporary storage of the character received
    BOOL bRxIntSetting;
    #if QSCI1_TX_INTERRUPT_DRIVEN
    BOOL bTxIntSetting;
    #endif

    while ( !( QSCI1_STAT & QSCI1_STAT_RDRF )  ) // Wait for a character/
    {
        // Wait for character to appear in buffer

        // TODO
        // CLEARWACHDOG();
    }

    #if QSCI1_TX_INTERRUPT_DRIVEN
    // Insure the TX interrupt is disabled before grabbing RX Interrupt status...
    //  (prevents us from restoring an invalid RX interrupt enable state)
    bTxIntSetting = QSCI1_CTRL1 & QSCI1_CTRL1_TEIE;            // Save off the current interrupt setting
    QSCI1_CTRL1 &= ~QSCI1_CTRL1_TEIE;
    asm( nop );
    asm( nop );
    #endif
    bRxIntSetting = QSCI1_CTRL1 & QSCI1_CTRL1_RFIE;             // Save off the current interrupt setting
    //DISABLE_QSCI0_RX_INTERRUPTS;
    QSCI1_CTRL1 &= ~QSCI1_CTRL1_RFIE;

    byChar = QSCI1_DATA;         // Read the next character from the buffer

    // Restore interrupt setting(s)
    QSCI1_CTRL1 |= bRxIntSetting;
    #if QSCI1_TX_INTERRUPT_DRIVEN
    QSCI1_CTRL1 |= bTxIntSetting;
    #endif

    return byChar;                                              // Return the character
}

//------------------------------------------------------------------------------
BYTE Qsci1GetRxBytes( void )
{
    // Return the number of bytes in the Rx buffer

    BYTE byNumOfBytes;                                          // Number of bytes in the RX Buffer
    BOOL bRxIntSetting;
    #if QSCI1_TX_INTERRUPT_DRIVEN
    BOOL bTxIntSetting;
    #endif

    #if QSCI1_TX_INTERRUPT_DRIVEN
    // Insure the TX interrupt is disabled before grabbing RX Interrupt status...
    //  (prevents us from restoring an invalid RX interrupt enable state)
    bTxIntSetting = QSCI1_CTRL1 & QSCI1_CTRL1_TEIE;           // Save off the current interrupt setting
    QSCI1_CTRL1 &= ~QSCI1_CTRL1_TEIE;
    asm( nop );
    asm( nop );
    #endif
    bRxIntSetting = QSCI1_CTRL1 & QSCI1_CTRL1_RFIE;            // Save off the current interrupt setting
    //DISABLE_QSCI0_RX_INTERRUPTS;
    QSCI1_CTRL1 &= ~QSCI1_CTRL1_RFIE;

    byNumOfBytes = ( ( QSCI1_CTRL2 & QSCI0_CTRL2_RFCNT ) >> 8 );

    // Restore interrupt setting(s)
    QSCI1_CTRL1 |= bRxIntSetting;
    #if QSCI1_TX_INTERRUPT_DRIVEN
    QSCI1_CTRL1 |= bTxIntSetting;
    #endif

    return byNumOfBytes;
}
#endif

//------------------------------------------------------------------------------
void Qsci1Init( void )
{
    // Initialize the QSCI0 controller
	volatile WORD st;
    // This connects the system clock to the serial peripheral
    //clock enabled in SIM init
	//ENABLE_SCI0_IPBUS_CLK;

    #if QSCI1_TX_INTERRUPT_DRIVEN
    byQSCI1TxBuffReadIndex  = 0;
    byQSCI1TxBuffWriteIndex = 0;
    byQSCI1TxStatus = 0;
    #endif

    // Hold QSCI0 in standby state during configuration
    QSCI1_CTRL1 |= QSCI1_CTRL1_RWU;       // 1=Standby state, 0=normal state

    // Configure baud rate
    QSCI1_RATE = SCI_1_SCIBR_INIT;

    // Configure the remainder of the QSCI0 module
    QSCI1_CTRL1 = SCI_1_SCICR_INIT;
    QSCI1_CTRL2 = SCI_1_SCICR2_INIT;

    // Release QSCI0 from reset state
    QSCI1_CTRL1 &= ~QSCI1_CTRL1_RWU;       // 1=Standby state, 0=normal state

    // Interrupts should be enabled by the application
#if QSCI1_TX_INTERRUPT_DRIVEN
    st = QSCI1_STAT;
    DEBUG_CONSOLE_INTERRUPT_PRIORITY;
#endif
}

#endif //QSCI1_ENABLED





