/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

//
// NOTE: The flash blocks within the devices CANNOT be accessed during write
//  erase operations.  Therefore, code execution CANNOT take place from within
//  the flash block that is to be erased or written to.  For this reason, this
//  module does NOT provide functions to write program flash locations.
//

//---------------------------
// STANDARD LIBRARY INCLUDES
//---------------------------


//----------------------
// APPLICATION INCLUDES
//----------------------
	#include "wct_callbacks.h"
	#include "wct_hal.h"
	#include "wct_hal_cfg.h"
	#include "peripheralInit.h"
	#include "flash.h"


//-----------------
// PRIVATE DEFINES
//-----------------


//-------------------
// PRIVATE VARIABLES
//-------------------


//-------------------
// PRIVATE FUNCTIONS
//-------------------


//------------------
// PUBLIC VARIABLES
//------------------


//------------------
// PUBLIC FUNCTIONS
//------------------

//------------------------------------------------------------------------------
//
// IMPORTANT NOTES:
//  1) Any attempt to write protected flash will cause an error.
//  3) Data Flash is 16-bits wide
//
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
void FlashControlInitialization()
{
    // Loaded from flash at reset, writable?
    FTFA_FPROT0 = FTFA_FPROT0_INIT;
    FTFA_FPROT1 = FTFA_FPROT1_INIT;
    FTFA_FPROT2 = FTFA_FPROT2_INIT;
    FTFA_FPROT3 = FTFA_FPROT3_INIT;
}

//------------------------------------------------------------------------------
void EraseProgramFlash( void )
{
    // NOTE: This command will perform a MASS erasure of all flash    

    // Wait until command complete flag is set
    while( !COMMAND_COMPLETE_INT_FLAG ){};

    // Clear all flags to allow next command
    CLEAR_PROTECTION_VIOLATION_FLAG;
    CLEAR_ACCESS_ERROR_FLAG;

    // Disable Interrupts
    DISABLE_CPU_INTERRUPTS;

    // Set command to erase the device, clear CCIF to execute
    FTFA_FCCOB0 = ERASE_ALL_BLOCKS;
    START_COMMAND_SEQUENCE;

}


//------------------------------------------------------------------------------
BOOL EraseDataFlashSector( BYTE bySector )
{
    // NOTE: Because this block of flash is separate from Program space, we can
    //       successfully execute the erasure from within flash.

    // NOTE: Flash command addressing is allocated on BYTE boundaries. This differs from
    // the WORD sized boundary default for the part.
    WORD wDataFlashAddress;
    BOOL bStatus = TRUE;

    if( bySector < NUM_DATA_FLASH_SECTORS )
    {
        // Create a pointer to any address within the sector
        wDataFlashAddress = DATA_FLASH_BASE_ADDRESS + ( FLASH_SECTOR_SIZE * bySector );

        // Wait until command complete flag is set
        while( !COMMAND_COMPLETE_INT_FLAG ){};

        // Clear all flags to allow next command
        CLEAR_PROTECTION_VIOLATION_FLAG;
        CLEAR_ACCESS_ERROR_FLAG;

        // Disable Interrupts
        DISABLE_CPU_INTERRUPTS;

        // Set command to erase a sector, write address c
        FTFA_FCCOB0 = ERASE_FLASH_SECTOR;

        // Write address in flash sector to be erased
        FTFA_FCCOB1 = 0;                                            // Flash address [23:16]
        FTFA_FCCOB2 = ( ( wDataFlashAddress & 0xFF00 ) >> 8 );      // Flash address [15:8]
        FTFA_FCCOB3 = ( wDataFlashAddress & 0x00FF );               // Flash address [7:0]

        // Clear CCIF to execute
        START_COMMAND_SEQUENCE;

        // Wait until command complete flag is set
        while( !COMMAND_COMPLETE_INT_FLAG ){};

        // Re-enable interrupts
        ENABLE_CPU_INTERRUPTS;

        if( PROTECTION_VIOLATION_FLAG || ACCESS_ERROR_FLAG )
        {
            // An error has occurred... clear the flags to allow future operations
            CLEAR_PROTECTION_VIOLATION_FLAG;
            CLEAR_ACCESS_ERROR_FLAG;
            bStatus = FALSE;
        }
    }
    else
    {
        // Illegal Data flash sector requested
        bStatus = FALSE;
    }

    return bStatus;
}


//------------------------------------------------------------------------------
void EraseDataFlash( void )
{
    // NOTE: Because the blocks of Data flash are separate from Program space, 
    //       we can successfully execute the erasure from within flash.

    BYTE byFlashSectorNum = 0;

    while( byFlashSectorNum < NUM_DATA_FLASH_SECTORS )
    {
        EraseDataFlashSector( byFlashSectorNum );
        byFlashSectorNum++;
    }

}



//------------------------------------------------------------------------------
BOOL WriteDataFlashLocation( WORD wAddress, DWORD dwData )
{
    // NOTE: Data can only be written to flash in DWORD blocks

    // NOTE: Flash command addressing is allocated on BYTE boundaries. This differs from
    // the WORD sized boundary default for the part.
    WORD wDataFlashAddress;
    BOOL bStatus = TRUE;

    // DO NOT attempt write if address is invalid
    if( wAddress < DATA_FLASH_SIZE )
    {
        // Create a pointer to the desired location
        wDataFlashAddress = DATA_FLASH_BASE_ADDRESS + wAddress;

        // Wait until command complete flag is set
        while( !COMMAND_COMPLETE_INT_FLAG ){};

        // Clear all flags to allow next command
        CLEAR_PROTECTION_VIOLATION_FLAG;
        CLEAR_ACCESS_ERROR_FLAG;

        // Disable Interrupts
        DISABLE_CPU_INTERRUPTS;

        // Set command to erase a page, write address c
        FTFA_FCCOB0 = PROGRAM_LONGWORD;

        // Write address in flash sector to be erased
        FTFA_FCCOB1 = 0;                                            // Flash address [23:16]
        FTFA_FCCOB2 = ( ( wDataFlashAddress & 0xFF00 ) >> 8 );     // Flash address [15:8]
        FTFA_FCCOB3 = ( wDataFlashAddress & 0x00FF );              // Flash address [7:0]

        // Write data to be saved to flash
        FTFA_FCCOB7 = ( ( dwData ) & 0xFF ) ;           // BYTE 1
        FTFA_FCCOB6 = ( ( dwData >> 8 ) & 0xFF );       // BYTE 2
        FTFA_FCCOB5 = ( ( dwData >> 16 ) & 0xFF );      // BYTE 3
        FTFA_FCCOB4 = ( ( dwData >> 24 ) & 0xFF );      // BYTE 4

        // Clear CCIF to execute
        START_COMMAND_SEQUENCE;

        // Wait until command complete flag is set
        while( !COMMAND_COMPLETE_INT_FLAG ){};

        // Re-enable interrupts
        ENABLE_CPU_INTERRUPTS;

        if( PROTECTION_VIOLATION_FLAG || ACCESS_ERROR_FLAG )
        {
            // An error has occurred... clear the flags to allow future operations
            CLEAR_PROTECTION_VIOLATION_FLAG;
            CLEAR_ACCESS_ERROR_FLAG;
            bStatus = FALSE;
        }
    }
    else
    {
        // Illegal address requested
        bStatus = FALSE;
    }

    return bStatus;
}


//------------------------------------------------------------------------------
WORD ReadDataFlashLocation( WORD wAddress )
{
    // NOTE: Flash command addressing is allocated on BYTE boundaries. This differs from
    // the WORD sized boundary default for the part.
    __pmem BYTE *pbyDataFlashAddress = (__pmem BYTE*)( DATA_FLASH_BASE_ADDRESS + wAddress );

    if( wAddress < DATA_FLASH_SIZE_IN_WORDS )
    {
        return ( *pbyDataFlashAddress );
    }
    else
    {
        return ( 0xFFFF );
    }
}



