/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

// General PWM manual operation and register description
//
// The PWM period and duty cycle is controlable differently for each mode of operation
// MODES OF OPERATION:
// 1. Center Aligned
//      Period is determined by the delta between the PWM_SMnINIT value and the PMW_SMnVAL1 value
//      Duty cycle is determined by PMW_SMnVAL2 and PMW_SMnVAL3 for PWMA; PMW_SMnVAL4 and PMW_SMnVAL5 for PWMB (both edges)
// 2. Edge Aligned
//      Period is determined by the delta between the PWM_SMnINIT value and the PMW_SMnVAL1 value
//      Duty cycle is determined by PMW_SMnVAL3 for PWMA; PMW_SMnVAL5 for PWMB (edges aligned at PWM_SMnINIT value
// 3. Phase Shifted
//      Period is determined by the delta between the PWM_SMnINIT value and the PMW_SMnVAL1 value
//      Duty cycle is determined by PMW_SMnVAL2 and PMW_SMnVAL3 for PWMA; PMW_SMnVAL4 and PMW_SMnVAL5 for PWMB (both edges)
//      Phase shift is the delta between PMW_SMnVAL2 and PMW_SMnVAL4
// 4. Double switching
//      Same as centered aligned. DBLPWM signal created by an XOR on PWMA and PWMB


// In manual mode of operation, the VAL registers trigger the outputs to swing high and low:
// INIT register sets the starting/reset point of the PWM counter (no output change)
// VAL0 register sets the midpoint of the period (no output change)
// VAL1 register sets the period (no output change)
// VAL2 register sets PWMA output start (sets output 'A' high on compare)
// VAL3 register sets PWMA output stop (sets output 'A' low on compare)
// VAL4 register sets PWMB output start (sets output 'B' high on compare)
// VAL5 register sets PWMB output stop (sets output 'B' low on compare)

// NOTE: In complementary mode, only PWMA is configured, PWMB is it's compliment with automatic deadtime insertion

#ifndef __PWM_H__

    #define __PWM_H__
    
    //----------
    // INCLUDES
    //----------

    //---------
    // DEFINES
    //---------

    // Timebase calculations
    #define PWM_CLOCK_SPEED                             ( IP_BUS_CLK )
    #define PWM_CLOCK_SPEED_IN_MHZ                      ( PWM_CLOCK_SPEED / 1000000 )
    #define PWM_CLOCK_PERIOD                            ( 1.0 / PWM_CLOCK_SPEED )
    #define PWM_DUTY_CYCLE_RESOLUTION                   ( PWM_CLOCK_PERIOD )
    #define PWM_PERIOD_RESOLUTION                       ( PWM_CLOCK_PERIOD * 8 )

    #define PWM_A_2_PRESCALER                           (1<<((PWM_A_2_CTRL_INIT&PWMA_SM2CTRL_PRSC)>>4))
    #define SOUND_PWM_CLK_PRESCALER                 PWM_A_2_PRESCALER
    #define SOUND_TIMER_CLOCK                               ( IP_BUS_CLK / SOUND_PWM_CLK_PRESCALER ) 

    // General definitions
    #define MAX_PWM_CHANNELS                            4
    #define MAX_DUTY_CYCLE_PERCENT                      10000       // Depicts 100% duty cycle (offers 2 significant digits of resolution)

    // Note: PRD interrupts are cleared by READING the status bit.
    #define ENABLE_PWM0_PERIOD_INTERRUPT                ( PWMA_SM0INTEN |= 0x0008 )
    #define DISABLE_PWM0_PERIOD_INTERRUPT               ( PWMA_SM0INTEN &= 0xFFF7 )
    #define PWM0_PERIOD_INTERRUPT_STATUS                ( PWMA_SM0STS & 0x0008 )
    #define ENABLE_PWM0_RELOAD_ERROR_INTERRUPT          ( PWMA_SM0INTEN |= PWMA_SM0INTEN_REIE )
    #define DISABLE_PWM0_RELOAD_ERROR_INTERRUPT         ( PWMA_SM0INTEN &= ~PWMA_SM0INTEN_REIE )
    #define ENABLE_PWM0_RELOAD_INTERRUPT                ( PWMA_SM0INTEN |= PWMA_SM0INTEN_RIE )
    #define DISABLE_PWM0_RELOAD_INTERRUPT               ( PWMA_SM0INTEN &= ~PWMA_SM0INTEN_RIE )


    #define ENABLE_PWM1_PERIOD_INTERRUPT                ( PWMA_SM1INTEN |= 0x0008 )
    #define DISABLE_PWM1_PERIOD_INTERRUPT               ( PWMA_SM1INTEN &= 0xFFF7 )
    #define PWM1_PERIOD_INTERRUPT_STATUS                ( PWMA_SM1STS & 0x0008 )
    #define ENABLE_PWM1_RELOAD_ERROR_INTERRUPT          ( PWMA_SM1INTEN |= PWMA_SM1INTEN_REIE )
    #define DISABLE_PWM1_RELOAD_ERROR_INTERRUPT         ( PWMA_SM1INTEN &= ~PWMA_SM1INTEN_REIE )
    #define ENABLE_PWM1_RELOAD_INTERRUPT                ( PWMA_SM1INTEN |= PWMA_SM1INTEN_RIE )
    #define DISABLE_PWM1_RELOAD_INTERRUPT               ( PWMA_SM1INTEN &= ~PWMA_SM1INTEN_RIE )


    #define ENABLE_PWM2_PERIOD_INTERRUPT                ( PWMA_SM2INTEN |= 0x0008 )
    #define DISABLE_PWM2_PERIOD_INTERRUPT               ( PWMA_SM2INTEN &= 0xFFF7 )
    #define PWM2_PERIOD_INTERRUPT_STATUS                ( PWMA_SM2STS & 0x0008 )
    #define ENABLE_PWM2_RELOAD_ERROR_INTERRUPT          ( PWMA_SM2INTEN |= PWMA_SM2INTEN_REIE )
    #define DISABLE_PWM2_RELOAD_ERROR_INTERRUPT         ( PWMA_SM2INTEN &= ~PWMA_SM2INTEN_REIE )
    #define ENABLE_PWM2_RELOAD_INTERRUPT                ( PWMA_SM2INTEN |= PWMA_SM2INTEN_RIE )
    #define DISABLE_PWM2_RELOAD_INTERRUPT               ( PWMA_SM2INTEN &= ~PWMA_SM2INTEN_RIE )

    #define ENABLE_PWM3_PERIOD_INTERRUPT                ( PWMA_SM3INTEN |= 0x0008 )
    #define DISABLE_PWM3_PERIOD_INTERRUPT               ( PWMA_SM3INTEN &= 0xFFF7 )
    #define PWM3_PERIOD_INTERRUPT_STATUS                ( PWMA_SM3STS & 0x0008 )
    #define ENABLE_PWM3_RELOAD_ERROR_INTERRUPT          ( PWMA_SM3INTEN |= PWMA_SM3INTEN_REIE )
    #define DISABLE_PWM3_RELOAD_ERROR_INTERRUPT         ( PWMA_SM3INTEN &= ~PWMA_SM3INTEN_REIE )
    #define ENABLE_PWM3_RELOAD_INTERRUPT                ( PWMA_SM3INTEN |= PWMA_SM3INTEN_RIE )
    #define DISABLE_PWM3_RELOAD_INTERRUPT               ( PWMA_SM3INTEN &= ~PWMA_SM3INTEN_RIE )

    // NOTE: PWM fault flags are cleared by writting a one to the flag
    #define CLEAR_PWM0_FAULT_FLAG                       ( PWMA_FSTS0 |= 0x0001 )
    #define CLEAR_PWM1_FAULT_FLAG                       ( PWMA_FSTS0 |= 0x0002 )
    #define CLEAR_PWM2_FAULT_FLAG                       ( PWMA_FSTS0 |= 0x0004 )
    #define CLEAR_PWM3_FAULT_FLAG                       ( PWMA_FSTS0 |= 0x0008 )


	#define CLEAR_PWM_FAULT_FLAG(x)                       ( PWMA_FSTS0 |= (0x01 << ((x)&0x03) ) )
	#define CHECK_PWM_FAULT_FLAG(x)                       ( PWMA_FSTS0 & ( 0x01 << ((x)&0x03) ) )
	
	// General Macros
	#define CALC_PERIOD_VALUE_FROM_FREQ( x )            ( PWM_CLOCK_SPEED / x )
	#define CALC_FREQ_FROM_PWMPRD_VALUE( x )            ( PWM_CLOCK_SPEED / x )

	#define SOUND_CALC_PERIOD_VALUE_FROM_FREQ( x )            ( SOUND_TIMER_CLOCK / x )
	#define SOUND_CALC_FREQ_FROM_PWMPRD_VALUE( x )            ( SOUND_TIMER_CLOCK / x )
	
	// The following macros utilize the MASK option to enable/disable PWM outputs
    #define MASK_PWM0A_OUTPUTS                            ( PWMA_MASK |=  PWMA_MASK_MASKA_0 )
    #define MASK_PWM0B_OUTPUTS                            ( PWMA_MASK |=  PWMA_MASK_MASKB_0 )
    #define MASK_ALL_PWM0_OUTPUTS                       ( PWMA_MASK |= ( PWMA_MASK_MASKA_0 | PWMA_MASK_MASKB_0 | PWMA_MASK_MASKX_0 ) )
    #define UNMASK_ALL_PWM0_OUTPUTS                     ( PWMA_MASK &= ~( PWMA_MASK_MASKA_0 | PWMA_MASK_MASKB_0 | PWMA_MASK_MASKX_0 ) )
    #define MASK_PWM1A_OUTPUTS                            ( PWMA_MASK |=  PWMA_MASK_MASKA_1 )
    #define MASK_PWM1B_OUTPUTS                            ( PWMA_MASK |=  PWMA_MASK_MASKB_1 )
    #define MASK_ALL_PWM1_OUTPUTS                       ( PWMA_MASK |= ( PWMA_MASK_MASKA_1 | PWMA_MASK_MASKB_1 | PWMA_MASK_MASKX_1 ) )
    #define UNMASK_ALL_PWM1_OUTPUTS                     ( PWMA_MASK &= ~( PWMA_MASK_MASKA_1 | PWMA_MASK_MASKB_1 | PWMA_MASK_MASKX_1 ) )
    #define MASK_PWM2A_OUTPUTS                            ( PWMA_MASK |=  PWMA_MASK_MASKA_2 )
    #define MASK_PWM2B_OUTPUTS                            ( PWMA_MASK |=  PWMA_MASK_MASKB_2 )
    #define MASK_ALL_PWM2_OUTPUTS                       ( PWMA_MASK |= ( PWMA_MASK_MASKA_2 | PWMA_MASK_MASKB_2 | PWMA_MASK_MASKX_2 ) )
    #define UNMASK_ALL_PWM2_OUTPUTS                     ( PWMA_MASK &= ~( PWMA_MASK_MASKA_2 | PWMA_MASK_MASKB_2 | PWMA_MASK_MASKX_2 ) )
    #define MASK_PWM3A_OUTPUTS                            ( PWMA_MASK |=  PWMA_MASK_MASKA_3 )
    #define MASK_PWM3B_OUTPUTS                            ( PWMA_MASK |=  PWMA_MASK_MASKB_3 )
    #define MASK_ALL_PWM3_OUTPUTS                       ( PWMA_MASK |= ( PWMA_MASK_MASKA_3 | PWMA_MASK_MASKB_3 | PWMA_MASK_MASKX_3 ) )
    #define UNMASK_ALL_PWM3_OUTPUTS                     ( PWMA_MASK &= ~( PWMA_MASK_MASKA_3 | PWMA_MASK_MASKB_3 | PWMA_MASK_MASKX_3 ) )
	// Macros to control FORCE_OUT event... NOTE: This is auto-clearing!
    #define ENABLE_PWM0_FORCE_OUT_EVENT                 ( PWMA_SM0CTRL2 |= ( 0x8000 | PWMA_SM0CTRL2_FORCE ) )
    #define ENABLE_PWM1_FORCE_OUT_EVENT                 ( PWMA_SM1CTRL2 |= ( 0x8000 | PWMA_SM1CTRL2_FORCE ) )
    #define ENABLE_PWM2_FORCE_OUT_EVENT                 ( PWMA_SM2CTRL2 |= ( 0x8000 | PWMA_SM2CTRL2_FORCE ) )
    #define ENABLE_PWM3_FORCE_OUT_EVENT                 ( PWMA_SM3CTRL2 |= ( 0x8000 | PWMA_SM3CTRL2_FORCE ) )
	
	
	
    #define ENABLE_PWM0A_OUTPUT_OVERRIDE                 { MASK_PWM0A_OUTPUTS; \
                                                          ENABLE_PWM0_FORCE_OUT_EVENT; }
    #define ENABLE_PWM0B_OUTPUT_OVERRIDE                 { MASK_PWM0B_OUTPUTS; \
                                                          ENABLE_PWM0_FORCE_OUT_EVENT; }
    #define ENABLE_PWM0_OUTPUT_OVERRIDE                 { MASK_ALL_PWM0_OUTPUTS; \
                                                          ENABLE_PWM0_FORCE_OUT_EVENT; }
    #define DISABLE_PWM0_OUTPUT_OVERRIDE                { UNMASK_ALL_PWM0_OUTPUTS; \
                                                          ENABLE_PWM0_FORCE_OUT_EVENT; }
    #define ENABLE_PWM1A_OUTPUT_OVERRIDE                 { MASK_PWM1A_OUTPUTS; \
                                                          ENABLE_PWM1_FORCE_OUT_EVENT; }
    #define ENABLE_PWM1B_OUTPUT_OVERRIDE                 { MASK_PWM1B_OUTPUTS; \
                                                          ENABLE_PWM1_FORCE_OUT_EVENT; }
    #define ENABLE_PWM1_OUTPUT_OVERRIDE                 { MASK_ALL_PWM1_OUTPUTS; \
                                                          ENABLE_PWM1_FORCE_OUT_EVENT; }
    #define DISABLE_PWM1_OUTPUT_OVERRIDE                { UNMASK_ALL_PWM1_OUTPUTS; \
                                                          ENABLE_PWM1_FORCE_OUT_EVENT; }
    #define ENABLE_PWM2A_OUTPUT_OVERRIDE                 { MASK_PWM2A_OUTPUTS; \
                                                          ENABLE_PWM2_FORCE_OUT_EVENT; }
    #define ENABLE_PWM2B_OUTPUT_OVERRIDE                 { MASK_PWM2B_OUTPUTS; \
                                                          ENABLE_PWM2_FORCE_OUT_EVENT; }
    #define ENABLE_PWM2_OUTPUT_OVERRIDE                 { MASK_ALL_PWM2_OUTPUTS; \
                                                          ENABLE_PWM2_FORCE_OUT_EVENT; }
    #define DISABLE_PWM2_OUTPUT_OVERRIDE                { UNMASK_ALL_PWM2_OUTPUTS; \
                                                          ENABLE_PWM2_FORCE_OUT_EVENT; }
    #define ENABLE_PWM3A_OUTPUT_OVERRIDE                 { MASK_PWM3A_OUTPUTS; \
                                                          ENABLE_PWM3_FORCE_OUT_EVENT; }
    #define ENABLE_PWM3B_OUTPUT_OVERRIDE                 { MASK_PWM3B_OUTPUTS; \
                                                          ENABLE_PWM3_FORCE_OUT_EVENT; }
    #define ENABLE_PWM3_OUTPUT_OVERRIDE                 { MASK_ALL_PWM3_OUTPUTS; \
                                                          ENABLE_PWM3_FORCE_OUT_EVENT; }
    #define DISABLE_PWM3_OUTPUT_OVERRIDE                { UNMASK_ALL_PWM3_OUTPUTS; \
                                                          ENABLE_PWM3_FORCE_OUT_EVENT; }

    #define ENABLE_PWM0                                 ( PWMA_MCTRL |= PWMA_MCTRL_RUN_0 )
    #define DISABLE_PWM0                                ( PWMA_MCTRL &= ~PWMA_MCTRL_RUN_0 )
    #define ENABLE_PWM1                                 ( PWMA_MCTRL |= PWMA_MCTRL_RUN_1 )
    #define DISABLE_PWM1                                ( PWMA_MCTRL &= ~PWMA_MCTRL_RUN_1 )
    #define ENABLE_PWM2                                 ( PWMA_MCTRL |= PWMA_MCTRL_RUN_2 )
    #define DISABLE_PWM2                                ( PWMA_MCTRL &= ~PWMA_MCTRL_RUN_2 )
    #define ENABLE_PWM3                                 ( PWMA_MCTRL |= PWMA_MCTRL_RUN_3 )
    #define DISABLE_PWM3                                ( PWMA_MCTRL &= ~PWMA_MCTRL_RUN_3 )

    #define ENABLE_PWMAB0_OUT                        (PWMA_OUTEN |= 0x0110)
    #define DISABLE_PWMAB0_OUT                        (PWMA_OUTEN &= ~0x0110)
    #define ENABLE_PWMAB1_OUT                        (PWMA_OUTEN |= 0x0220)
    #define DISABLE_PWMAB1_OUT                        (PWMA_OUTEN &= ~0x0220)
    #define ENABLE_PWMAB2_OUT                        (PWMA_OUTEN |= 0x0440)
    #define DISABLE_PWMAB2_OUT                        (PWMA_OUTEN &= ~0x0440)
    #define ENABLE_PWMAB3_OUT                        (PWMA_OUTEN |= 0x0880)
    #define DISABLE_PWMAB3_OUT                        (PWMA_OUTEN &= ~0x0880)

    #define ENABLE_PWMA2_OUT                        (PWMA_OUTEN |= 0x0400)
    #define DISABLE_PWMA2_OUT                        (PWMA_OUTEN &= ~0x0400)
    #define ENABLE_PWMB2_OUT                        (PWMA_OUTEN |= 0x0040)
    #define DISABLE_PWMB2_OUT                        (PWMA_OUTEN &= ~0x0040)

 // Note: PRD interrupts are cleared by writing a '1' to the associated status bit.
    #define CLEAR_PWM0_RELOAD_ERROR_INTERRUPT_STATUS    ( PWMA_SM0STS |= PWMA_SM0STS_REF )
    #define CLEAR_PWM0_RELOAD_INTERRUPT_STATUS          ( PWMA_SM0STS |= PWMA_SM0STS_RF )
    #define CLEAR_PWM1_RELOAD_ERROR_INTERRUPT_STATUS    ( PWMA_SM1STS |= PWMA_SM0STS_REF )
    #define CLEAR_PWM1_RELOAD_INTERRUPT_STATUS          ( PWMA_SM1STS |= PWMA_SM0STS_RF )
    #define CLEAR_PWM2_RELOAD_ERROR_INTERRUPT_STATUS    ( PWMA_SM2STS |= PWMA_SM0STS_REF )
    #define CLEAR_PWM2_RELOAD_INTERRUPT_STATUS          ( PWMA_SM2STS |= PWMA_SM0STS_RF )
    #define CLEAR_PWM3_RELOAD_ERROR_INTERRUPT_STATUS    ( PWMA_SM3STS |= PWMA_SM0STS_REF )
    #define CLEAR_PWM3_RELOAD_INTERRUPT_STATUS          ( PWMA_SM3STS |= PWMA_SM0STS_RF )

    #define PWM0_RELOAD_ERROR_INTERRUPT_STATUS          ( PWMA_SM0STS & PWMA_SM0STS_REF )
    #define PWM0_RELOAD_INTERRUPT_STATUS                ( PWMA_SM0STS & PWMA_SM0STS_RF )
    #define PWM1_RELOAD_ERROR_INTERRUPT_STATUS          ( PWMA_SM1STS & PWMA_SM0STS_REF )
    #define PWM1_RELOAD_INTERRUPT_STATUS                ( PWMA_SM1STS & PWMA_SM0STS_RF )
    #define PWM2_RELOAD_ERROR_INTERRUPT_STATUS          ( PWMA_SM2STS & PWMA_SM0STS_REF )
    #define PWM2_RELOAD_INTERRUPT_STATUS                ( PWMA_SM2STS & PWMA_SM0STS_RF )
    #define PWM3_RELOAD_ERROR_INTERRUPT_STATUS          ( PWMA_SM3STS & PWMA_SM0STS_REF )
    #define PWM3_RELOAD_INTERRUPT_STATUS                ( PWMA_SM3STS & PWMA_SM0STS_RF )


#if defined (PWM_A_0_CTRL_INIT) || \
    defined (PWM_A_0_CTRL2_INIT) || \
    defined (PWM_A_0_INIT_INIT) || \
    defined (PWM_A_0_VAL1_INIT) || \
    defined (PWM_A_0_FRACVAL1_INIT) || \
    defined (PWM_A_0_FRCTRL_INIT) || \
    defined (PWM_A_0_OCTRL_INIT) || \
    defined (PWM_A_0_VAL0_INIT) || \
    defined (PWM_A_0_VAL2_INIT) || \
    defined (PWM_A_0_VAL3_INIT) || \
    defined (PWM_A_0_VAL4_INIT) || \
    defined (PWM_A_0_VAL5_INIT) || \
    defined (PWM_A_0_FRACVAL2_INIT) || \
    defined (PWM_A_0_FRACVAL3_INIT) || \
    defined (PWM_A_0_FRACVAL4_INIT) || \
    defined (PWM_A_0_FRACVAL5_INIT) || \
    defined (PWM_A_0_TCTRL_INIT) || \
    defined (PWM_A_0_DTCNT0_INIT) || \
    defined (PWM_A_0_DTCNT1_INIT) || \
    defined (PWM_A_0_INTEN_INIT) || \
    defined (PWM_A_0_DISMAP_INIT) || \
    defined (PWM_A_0_DISMAP0_INIT) || \
    defined (PWM_A_0_DISMAP1_INIT)
#define PWM0_ENABLED  1
#endif

#if defined (PWM_A_1_CTRL_INIT) || \
    defined (PWM_A_1_CTRL2_INIT) || \
    defined (PWM_A_1_INIT_INIT) || \
    defined (PWM_A_1_VAL1_INIT) || \
    defined (PWM_A_1_FRACVAL1_INIT) || \
    defined (PWM_A_1_FRCTRL_INIT) || \
    defined (PWM_A_1_OCTRL_INIT) || \
    defined (PWM_A_1_VAL0_INIT) || \
    defined (PWM_A_1_VAL2_INIT) || \
    defined (PWM_A_1_VAL3_INIT) || \
    defined (PWM_A_1_VAL4_INIT) || \
    defined (PWM_A_1_VAL5_INIT) || \
    defined (PWM_A_1_FRACVAL2_INIT) || \
    defined (PWM_A_1_FRACVAL3_INIT) || \
    defined (PWM_A_1_FRACVAL4_INIT) || \
    defined (PWM_A_1_FRACVAL5_INIT) || \
    defined (PWM_A_1_TCTRL_INIT) || \
    defined (PWM_A_1_DTCNT0_INIT) || \
    defined (PWM_A_1_DTCNT1_INIT) || \
    defined (PWM_A_1_INTEN_INIT) || \
    defined (PWM_A_1_DISMAP_INIT) || \
    defined (PWM_A_1_DISMAP0_INIT) || \
    defined (PWM_A_1_DISMAP1_INIT)
#define PWM1_ENABLED  1
#endif

#if defined (PWM_A_2_CTRL_INIT) || \
    defined (PWM_A_2_CTRL2_INIT) || \
    defined (PWM_A_2_INIT_INIT) || \
    defined (PWM_A_2_VAL1_INIT) || \
    defined (PWM_A_2_FRACVAL1_INIT) || \
    defined (PWM_A_2_FRCTRL_INIT) || \
    defined (PWM_A_2_OCTRL_INIT) || \
    defined (PWM_A_2_VAL0_INIT) || \
    defined (PWM_A_2_VAL2_INIT) || \
    defined (PWM_A_2_VAL3_INIT) || \
    defined (PWM_A_2_VAL4_INIT) || \
    defined (PWM_A_2_VAL5_INIT) || \
    defined (PWM_A_2_FRACVAL2_INIT) || \
    defined (PWM_A_2_FRACVAL3_INIT) || \
    defined (PWM_A_2_FRACVAL4_INIT) || \
    defined (PWM_A_2_FRACVAL5_INIT) || \
    defined (PWM_A_2_TCTRL_INIT) || \
    defined (PWM_A_2_DTCNT0_INIT) || \
    defined (PWM_A_2_DTCNT1_INIT) || \
    defined (PWM_A_2_INTEN_INIT) || \
    defined (PWM_A_2_DISMAP_INIT) || \
    defined (PWM_A_2_DISMAP0_INIT) || \
    defined (PWM_A_2_DISMAP1_INIT)
#define PWM2_ENABLED  1
#endif

#if defined (PWM_A_3_CTRL_INIT) || \
    defined (PWM_A_3_CTRL2_INIT) || \
    defined (PWM_A_3_INIT_INIT) || \
    defined (PWM_A_3_VAL1_INIT) || \
    defined (PWM_A_3_FRACVAL1_INIT) || \
    defined (PWM_A_3_FRCTRL_INIT) || \
    defined (PWM_A_3_OCTRL_INIT) || \
    defined (PWM_A_3_VAL0_INIT) || \
    defined (PWM_A_3_VAL2_INIT) || \
    defined (PWM_A_3_VAL3_INIT) || \
    defined (PWM_A_3_VAL4_INIT) || \
    defined (PWM_A_3_VAL5_INIT) || \
    defined (PWM_A_3_FRACVAL2_INIT) || \
    defined (PWM_A_3_FRACVAL3_INIT) || \
    defined (PWM_A_3_FRACVAL4_INIT) || \
    defined (PWM_A_3_FRACVAL5_INIT) || \
    defined (PWM_A_3_TCTRL_INIT) || \
    defined (PWM_A_3_DTCNT0_INIT) || \
    defined (PWM_A_3_DTCNT1_INIT) || \
    defined (PWM_A_3_INTEN_INIT) || \
    defined (PWM_A_3_DISMAP_INIT) || \
    defined (PWM_A_3_DISMAP0_INIT) || \
    defined (PWM_A_3_DISMAP1_INIT)
#define PWM3_ENABLED  1
#endif

    //----------
    // TYPEDEFS
    //----------


    //-----------
    // VARIABLES
    //-----------


    //-----------
    // FUNCTIONS
    //-----------
void Pwm0Init( WORD wDeadTimeNs );
void Pwm1Init( WORD wDeadTimeNs );
void Pwm2Init( WORD wDeadTimeNs );
void Pwm3Init( WORD wDeadTimeNs );

    // Standard function prototypes
    void PwmInit( WORD wDeadTimeNs );
    void PwmFrequencyUpdate( BYTE byChannel, DWORD dwNewFrequency, WORD wDutyCyclePercent );
    void PwmDutyCycleUpdate( BYTE byChannel, WORD wDutyCyclePercent );

#endif
