/*******************************************************************************
*
* Copyright 2004-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

#include "defines.h"
#include "wct_debug.h"
//#include "wct_lib_cfg.h"
#include "wct_hal_cfg.h"
#include "wct_lib.h"
#include "wct_callbacks.h"
#include "wct_hal.h"
#include "peripheralInit.h"
#include "flash.h"
#include "adc.h"

#define WCTTX_DEBUG_BUFFER_SIZE 20

#define WCTTX_DEBUG_CMD_STATE_IDLE        0     //Sets PC when command is finished
#define WCTTX_DEBUG_CMD_STATE_RECEIVED    1     //Sets PC when new command is send to board
#define WCTTX_DEBUG_CMD_STATE_PROGRESS    2     //Sets Board, when command is in progress
#define WCTTX_DEBUG_CMD_STATE_DONE        3     //Set board, when command is done and buffer has result

#define WCTTX_DEBUG_CMD_STATUS_OK         0
#define WCTTX_DEBUG_CMD_STATUS_FAILED     0x80
#define WCTTX_DEBUG_CMD_STATUS_FAILED_FLASHWRITE 0x81
#define WCTTX_DEBUG_CMD_STATUS_FAILED_UNCALIBRATED 0x82
#define WCTTX_DEBUG_CMD_STATUS_FAILED_INV_PARAM 0x83

#define WCTTX_DEBUG_CMD_NVM_FLASH         0x10
#define WCTTX_DEBUG_CMD_DEBUG_MODE        0x12
#define WCTTX_DEBUG_CMD_GET_INPUT_VOLTAGE 0x13
#define WCTTX_DEBUG_CMD_RUN_RAILVOLTAGE_CALIBRATION 0x14
#define WCTTX_DEBUG_CMD_READ_RAIL_VOLTAGE_CONTROL 0x15
#define WCTTX_DEBUG_CMD_COIL_CONTROL      0x16
#define WCTTX_DEBUG_CMD_COIL_DISCHARGE    0x17
#define WCTTX_DEBUG_CMD_COIL_DUTY_CYCLE   0x18

static DWORD dwFreqValue=0;
static BOOL bCoilControl=WCT_DISABLE;
static BOOL bDebugStatus = FALSE;     //Build for normalisation GUI fault when reading

WORD  wcttxDebugStatus = 0;
BYTE  wcttxDebugBuffer[WCTTX_DEBUG_BUFFER_SIZE];


static BYTE DiagnosticWriteNVM( void )
{
#if FLASH_PAGE_SIZE_IN_WORDS /* 82xx flash driver */
    WORD* pStructParam = (WORD*) &NvmParams.SystemParams;
#else
    DWORD* pStructParam = (DWORD*) &NvmParams.SystemParams;
#endif
    WORD wDFlashAddress = 0;
    WORD i;
    WORD wNvmParameterSize;
    BOOL bSuccess = TRUE;
    BYTE byDeviceId;
    BYTE byCalcChecksum;

    //update System params checksum
    byCalcChecksum = SystemComputeDataBlockChecksum( (BYTE *)&NvmParams.SystemParams, sizeof(SYSTEM_PARAMS) );
    NvmParams.SystemParams.ChecksumParams.byChecksum = byCalcChecksum;

    //update Coil params checksum
    for( byDeviceId = 0; byDeviceId < NUM_DEVICES; byDeviceId++ )
    {
    	byCalcChecksum = SystemComputeDataBlockChecksum( (BYTE *)&NvmParams.OpParams[ byDeviceId ], sizeof(OP_PARAMS) );
    	NvmParams.OpParams[ byDeviceId ].ChecksumParams.byChecksum = byCalcChecksum; 
    }
    //update calibration checksum
    byCalcChecksum = SystemComputeDataBlockChecksum( (BYTE *)&NvmParams.CalParams, sizeof(CAL_PARAMS) );
    NvmParams.CalParams.ChecksumParams.byChecksum =byCalcChecksum;
    
    wNvmParameterSize = sizeof( NVM_PARAMS ); 

    // Divide parameter size by 4 to calculate # of DWORDS
#if FLASH_PAGE_SIZE_IN_WORDS /* 82xx flash driver */
    wNvmParameterSize >>= 1;
#else
    wNvmParameterSize >>= 2;
#endif

    // Erase the entire data flash
    EraseDataFlash();

    // Reprogram with all valid parameter data
    for( i = 0; i < wNvmParameterSize; i++ )
    { 
#if FLASH_PAGE_SIZE_IN_WORDS /* 82xx flash driver */
        if( !WriteDataFlashLocation( wDFlashAddress, (DWORD) (*pStructParam) ) )
#else
        if( !WriteDataFlashLocation( wDFlashAddress, *pStructParam ) )
#endif
        {
            bSuccess = FALSE;
        }
        // Data flash is written in DWORD increments
#if FLASH_PAGE_SIZE_IN_WORDS /* 82xx flash driver */
        wDFlashAddress++;
#else
        wDFlashAddress += 4;
#endif
        pStructParam++;
    }

    if( !bSuccess )
    {
        return WCTTX_DEBUG_CMD_STATUS_FAILED_FLASHWRITE;
    }
    return WCTTX_DEBUG_CMD_STATUS_OK;
}

//------------------------------------------------------------------------------
static BYTE DiagnosticsPLDParameters( BYTE* pBuff )
{
    RUNTIME_PARAMS* pRunTimeParams;
    FOD_CHARACTERIZATION_PARAMS* pPwrLossCharParams;
    SDWORD sdwPPT;
    BYTE byCoilId;
    BYTE byDeviceId;
    WORD* pwBuff=(WORD*) pBuff;
    WORD wInputVol;

    byCoilId = pBuff[0];

    if( NUM_COILS > byCoilId )
    {
        // Calculate pointers
        byDeviceId = ( byCoilId / gStaticConf.byNumCoilsPerDevice );
        pRunTimeParams = &RunTimeParams[ byDeviceId ];
        pRunTimeParams->byDeviceId = byDeviceId;
        pRunTimeParams->byActiveCoilId = byCoilId;
        pRunTimeParams->byCoilIdIndex = ( byCoilId % NUM_COILS_PER_DEVICE );
        pPwrLossCharParams = &NvmParams.CalParams.PowerLossParams[ pRunTimeParams->byDeviceId ].FodCharacterizationParams[ pRunTimeParams->byCoilIdIndex ];

        if(bDebugStatus)
        {
        // Take ADC samples to fill the collection buffer
        for( pRunTimeParams->wFodBufferIndex = 0, pRunTimeParams->wFodSampleCount = 0; pRunTimeParams->wFodBufferIndex < MAX_POWER_LOSS_SAMPLE_HISTORY; )
        {
            if( pRunTimeParams->wFodSampleCount == 0 )
            {
                // First write to buffer location, just save latest data
                pRunTimeParams->wCoilCurrentHistory[ pRunTimeParams->wFodBufferIndex ] = AnalogIoGetCoilCurrentLowValue(pRunTimeParams->byActiveCoilId, WCT_AnalogIoGetCoilCurrent( pRunTimeParams->byActiveCoilId, CURRENT_REPORT_ADC_COUNTS )); 
                pRunTimeParams->wInputCurrentHistory[ pRunTimeParams->wFodBufferIndex ] = WCT_AnalogIoGetInputCurrent( pRunTimeParams->byActiveCoilId, CURRENT_REPORT_ADC_COUNTS );
                pRunTimeParams->wInputVoltageHistory[ pRunTimeParams->wFodBufferIndex ] = WCT_AnalogIoGetPowerSupplyVoltage( VOLTAGE_REPORT_ADC_COUNTS );
            }
            else
            {
                // Incremental write to buffer location, accumulate the data
                pRunTimeParams->wCoilCurrentHistory[ pRunTimeParams->wFodBufferIndex ] += AnalogIoGetCoilCurrentLowValue(pRunTimeParams->byActiveCoilId, WCT_AnalogIoGetCoilCurrent( pRunTimeParams->byActiveCoilId, CURRENT_REPORT_ADC_COUNTS )); 
                pRunTimeParams->wInputCurrentHistory[ pRunTimeParams->wFodBufferIndex ] += WCT_AnalogIoGetInputCurrent( pRunTimeParams->byActiveCoilId, CURRENT_REPORT_ADC_COUNTS );
                pRunTimeParams->wInputVoltageHistory[ pRunTimeParams->wFodBufferIndex ] += WCT_AnalogIoGetPowerSupplyVoltage( VOLTAGE_REPORT_ADC_COUNTS );
            }

            // Manage sample counter
            if( pRunTimeParams->wFodSampleCount == 3 )
            {
                // Done with this buffer location
                pRunTimeParams->wFodSampleCount = 0;
                pRunTimeParams->wFodBufferIndex++;
            }
            else
            {
                pRunTimeParams->wFodSampleCount++;
            }
        }

        // Prepare for calculations
        pRunTimeParams->wFodBufferIndex = 0;
        pRunTimeParams->byFodWindowSize = MAX_POWER_LOSS_SAMPLE_HISTORY;
        pRunTimeParams->byFodWindowOffset = 0;

        // Calculate the Input Power and Losses
        WCT_CalculateInputPower( pRunTimeParams, FALSE );
        }
        sdwPPT = pRunTimeParams->dwCalculatedInputPower;
        sdwPPT -= pRunTimeParams->swTxInternalLoss;
        
        //--------------------Calculate input voltage when isn't debug mode-----------------//
        wInputVol = WCT_AnalogIoGetPowerSupplyVoltage( VOLTAGE_REPORT_ADC_COUNTS );
        wInputVol = WCT_ConvertInVolAdcToMilliVolts(wInputVol);
        //----------------------------------------------------------------------------------//

        // Report the results
        //RunTimeDebugPutString( "\r\nPLD " );
        if(pBuff[1])
        {
            pwBuff[0] = pRunTimeParams->swQuadTerm;
            pwBuff[1] = pRunTimeParams->swLinearTerm;
            pwBuff[2] = pPwrLossCharParams->swConstantCoefficient;
            pwBuff[3] = pPwrLossCharParams->swPowerLossCalibrationOffset;
        }
        else
        {
            pwBuff[0] = pRunTimeParams->wMeasuredInputVoltage;
            pwBuff[1] = pRunTimeParams->dwMeasuredInputCurrent;
            pwBuff[2] = pRunTimeParams->wMeasuredCoilCurrent;
            pwBuff[3] = pRunTimeParams->dwCalculatedInputPower&0xffff;
            //pwBuff[4] = (pRunTimeParams->dwCalculatedInputPower>>16)&0xffff;
            pwBuff[4] = pRunTimeParams->swTxInternalLoss;
            pwBuff[5] = wInputVol;
            pwBuff[6] = pRunTimeParams->wPreviousAnalogPingAdcCounts[0];
        }
/*        RunTimeDebugPutSignedDecWord( pRunTimeParams->swQuadTerm );
        RunTimeDebugPutString( ", " );
        RunTimeDebugPutSignedDecWord( pRunTimeParams->swLinearTerm );
        RunTimeDebugPutString( ", " );
        RunTimeDebugPutSignedDecWord( pPwrLossCharParams->swConstantCoefficient );
        RunTimeDebugPutString( " (" );
        RunTimeDebugPutSignedDecWord( pPwrLossCharParams->swPowerLossCalibrationOffset );
        DebugFlushBuffer();

        RunTimeDebugPutString( "mW offset)\r\nIcoil: " );
        RunTimeDebugPutDoubleWordValue( pRunTimeParams->wMeasuredCoilCurrent );
        RunTimeDebugPutString( "mA, Vinput: " );
        RunTimeDebugPutDoubleWordValue( pRunTimeParams->wMeasuredInputVoltage );
        RunTimeDebugPutString( "mV, Iinput: " );
        RunTimeDebugPutDoubleWordValue( pRunTimeParams->dwMeasuredInputCurrent );
        RunTimeDebugPutString( "mA" );
        DebugFlushBuffer();*/
        if( ( pRunTimeParams->wCoilCurrentCalibration == NOMINAL_CALIBRATION_SCALE_FACTOR ) &&
            ( pRunTimeParams->wInputCurrentCalibration == NOMINAL_CALIBRATION_SCALE_FACTOR ) &&
            ( pRunTimeParams->wInputVoltageCalibration == NOMINAL_CALIBRATION_SCALE_FACTOR ) )
        {
            return WCTTX_DEBUG_CMD_STATUS_FAILED_UNCALIBRATED;
        }
        return WCTTX_DEBUG_CMD_STATUS_OK;
        /*
        RunTimeDebugPutString( "\r\nPinput: " );
        RunTimeDebugPutDoubleWordValue( pRunTimeParams->dwCalculatedInputPower );
        RunTimeDebugPutString( "mW\r\nTxInternalLoss: " );
        RunTimeDebugPutDoubleWordValue( pRunTimeParams->swTxInternalLoss );
        RunTimeDebugPutString( "mW\r\nPPT: " );
        RunTimeDebugPutSignedDecWord( sdwPPT );
        RunTimeDebugPutString( "mW\r\n\r\n" );*/
    }
}

void WctDebugProcess(void)
{
  BYTE byStatus=0;
  
  if(wcttxDebugStatus == WCTTX_DEBUG_CMD_STATE_RECEIVED)
  {
    switch(wcttxDebugBuffer[0])
    {
      case WCTTX_DEBUG_CMD_NVM_FLASH:
      {
        //todo apply init according the wcttxDebugBuffer[2], in gui will be 3 chechboxes, which defines, what memory block should be ininialized
        //probably KO, because write does not allow this concept
        if(wcttxDebugBuffer[2] == 0)
        {
            byStatus = WCT_InitSystemParams((WORD*) SYSTEM_PARAMS_FLASH_BASE_ADDRESS, (WORD*)&NvmParams.SystemParams, sizeof( SYSTEM_PARAMS ));
            if (byStatus == ERROR_CODE_SUCCESS)
                byStatus = WCT_InitOpParams((WORD*) OP_PARAMS_FLASH_BASE_ADDRESS_FIRST_DEVICE, (WORD*)&NvmParams.OpParams, sizeof( OP_PARAMS ));
            if (byStatus == ERROR_CODE_SUCCESS)
                byStatus = WCT_InitCalParams((WORD*) CAL_PARAMS_FLASH_BASE_ADDRESS, (WORD*)&NvmParams.CalParams, sizeof( CAL_PARAMS ));
            WCT_Run();
        }
        else
        {
          byStatus = DiagnosticWriteNVM();
        }
        break;
      }
      case WCTTX_DEBUG_CMD_DEBUG_MODE:
      {
          BYTE st = (wcttxDebugBuffer[2]) ? 0 : 1;
          bDebugStatus = st;
          WCT_OpStateDebugMode(&st);
          WCT_Run();
        break;
      }
      case WCTTX_DEBUG_CMD_GET_INPUT_VOLTAGE:
      {
    	  RUNTIME_PARAMS* pRunTimeParams;
    	  pRunTimeParams = &RunTimeParams[ 0 ];
          if((bCoilControl==WCT_ENABLE) && bDebugStatus)
          {
              AdcWriteChannel(ADC_CHANNEL_COIL_CURR_SENSE_C1, DDM_SetBestTriggerPos(0, 0,pRunTimeParams));
          }
          
          byStatus = DiagnosticsPLDParameters(&wcttxDebugBuffer[3]);
        break;
      }
      case WCTTX_DEBUG_CMD_COIL_CONTROL:
      {
         WORD temp = (wcttxDebugBuffer[6])| (wcttxDebugBuffer[7]<<8);
          BYTE byCoilId = wcttxDebugBuffer[3];

          dwFreqValue = (wcttxDebugBuffer[4] | (wcttxDebugBuffer[5]<<8))&0xffff;
          dwFreqValue |= (((DWORD)temp)<<16);
          if (wcttxDebugBuffer[2] == 0)
          {
              if( dwFreqValue < 999999 )
              {
                byStatus = WCT_CoilDischarge( byCoilId, 0 );
                if(byStatus == ERROR_CODE_SUCCESS)
                  byStatus = WCT_CoilFreqSet( byCoilId, dwFreqValue );
                if(byStatus == ERROR_CODE_SUCCESS)
                {
                  byStatus = WCT_CoilControl( byCoilId, WCT_ENABLE );
                  bCoilControl=WCT_ENABLE;
                }
              }
              else
              {
                byStatus = WCTTX_DEBUG_CMD_STATUS_FAILED_INV_PARAM;
              }
          }
          else
          {
            byStatus = WCT_CoilControl( byCoilId, WCT_DISABLE );
            bCoilControl=WCT_DISABLE;
          }
          break;
      }
      case WCTTX_DEBUG_CMD_COIL_DISCHARGE:
      {
          BYTE byCoilId = wcttxDebugBuffer[3];
          BYTE byCoilDischarge = wcttxDebugBuffer[2] ? 0 : 1;
          byStatus = WCT_CoilControl( byCoilId, WCT_DISABLE ); 
          bCoilControl=WCT_DISABLE;
          if(byStatus == ERROR_CODE_SUCCESS)
            byStatus = WCT_CoilDischarge( byCoilId, byCoilDischarge );
          break;
      }
      case WCTTX_DEBUG_CMD_COIL_DUTY_CYCLE:
      {
          WORD wDutyCyclePercent = (wcttxDebugBuffer[4] | (wcttxDebugBuffer[5]<<8));
          BYTE byCoilId = wcttxDebugBuffer[3];
          if (wcttxDebugBuffer[2] == 0)
          {
            if (wDutyCyclePercent <= 10000)
              WCT_CoilDutyCycleSet( byCoilId, wDutyCyclePercent );
            else
              byStatus = WCTTX_DEBUG_CMD_STATUS_FAILED_INV_PARAM;
          }
          else
          {
            byStatus = WCT_CoilControl( byCoilId, WCT_DISABLE );
            bCoilControl=WCT_DISABLE;
          }
          break;
      }
      
      default:
      {
          byStatus = 0xfe; //invalid command
      }
    }
    wcttxDebugStatus = WCTTX_DEBUG_CMD_STATE_PROGRESS;
    wcttxDebugStatus |= byStatus<<8;
  }
}
